# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/monitoring.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import audit_pb2 as chalk_dot_auth_dot_v1_dot_audit__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.utils.v1 import sensitive_pb2 as chalk_dot_utils_dot_v1_dot_sensitive__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n chalk/server/v1/monitoring.proto\x12\x0f\x63halk.server.v1\x1a\x19\x63halk/auth/v1/audit.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1e\x63halk/utils/v1/sensitive.proto\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\xd3\x02\n\x17PagerDutyEventV2Payload\x12\x18\n\x07summary\x18\x01 \x01(\tR\x07summary\x12=\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\ttimestamp\x88\x01\x01\x12>\n\x08severity\x18\x03 \x01(\x0e\x32".chalk.server.v1.PagerDutySeverityR\x08severity\x12\x16\n\x06source\x18\x04 \x01(\tR\x06source\x12!\n\tcomponent\x18\x05 \x01(\tH\x01R\tcomponent\x88\x01\x01\x12\x19\n\x05group\x18\x06 \x01(\tH\x02R\x05group\x88\x01\x01\x12\x19\n\x05\x63lass\x18\x07 \x01(\tH\x03R\x05\x63lass\x88\x01\x01\x42\x0c\n\n_timestampB\x0c\n\n_componentB\x08\n\x06_groupB\x08\n\x06_class"L\n\x14PagerDutyEventV2Link\x12\x12\n\x04href\x18\x01 \x01(\tR\x04href\x12\x17\n\x04text\x18\x02 \x01(\tH\x00R\x04text\x88\x01\x01\x42\x07\n\x05_text"j\n\x15PagerDutyEventV2Image\x12\x10\n\x03src\x18\x01 \x01(\tR\x03src\x12\x17\n\x04href\x18\x02 \x01(\tH\x00R\x04href\x88\x01\x01\x12\x15\n\x03\x61lt\x18\x03 \x01(\tH\x01R\x03\x61lt\x88\x01\x01\x42\x07\n\x05_hrefB\x06\n\x04_alt"\xc9\x03\n\x10PagerDutyEventV2\x12\x42\n\x07payload\x18\x01 \x01(\x0b\x32(.chalk.server.v1.PagerDutyEventV2PayloadR\x07payload\x12\x1f\n\x0brouting_key\x18\x02 \x01(\tR\nroutingKey\x12H\n\x0c\x65vent_action\x18\x03 \x01(\x0e\x32%.chalk.server.v1.PagerDutyEventActionR\x0b\x65ventAction\x12 \n\tdedup_key\x18\x04 \x01(\tH\x00R\x08\x64\x65\x64upKey\x88\x01\x01\x12\x1b\n\x06\x63lient\x18\x05 \x01(\tH\x01R\x06\x63lient\x88\x01\x01\x12"\n\nclient_url\x18\x06 \x01(\tH\x02R\tclientUrl\x88\x01\x01\x12;\n\x05links\x18\x07 \x03(\x0b\x32%.chalk.server.v1.PagerDutyEventV2LinkR\x05links\x12>\n\x06images\x18\x08 \x03(\x0b\x32&.chalk.server.v1.PagerDutyEventV2ImageR\x06imagesB\x0c\n\n_dedup_keyB\t\n\x07_clientB\r\n\x0b_client_url"\xa5\x01\n\x14PagerDutyIntegration\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x18\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\x08R\x07\x64\x65\x66\x61ult\x12\x1a\n\x05token\x18\x04 \x01(\tB\x04\xd8\xa1\'\x01R\x05token\x12%\n\x0e\x65nvironment_id\x18\x05 \x01(\tR\renvironmentIdB\x07\n\x05_name"\x9c\x02\n\x15IncidentIoIntegration\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1a\n\x05token\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\x05token\x12%\n\x0e\x65nvironment_id\x18\x03 \x01(\tR\renvironmentId\x12\x17\n\x04name\x18\x04 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1f\n\tsource_id\x18\x05 \x01(\tB\x02\x18\x01R\x08sourceId\x12%\n\x0csource_token\x18\x06 \x01(\tB\x02\x18\x01R\x0bsourceToken\x12\x1f\n\x0bseverity_id\x18\x07 \x01(\tR\nseverityId\x12%\n\x0eseverity_token\x18\x08 \x01(\tR\rseverityTokenB\x07\n\x05_name"\xa1\x02\n\x11IncidentIoEventV2\x12\x19\n\x08route_id\x18\x01 \x01(\tR\x07routeId\x12%\n\x0broute_token\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\nrouteToken\x12 \n\tdedup_key\x18\x03 \x01(\tH\x00R\x08\x64\x65\x64upKey\x88\x01\x01\x12"\n\nsource_url\x18\x04 \x01(\tH\x01R\tsourceUrl\x88\x01\x01\x12%\n\x0b\x64\x65scription\x18\x05 \x01(\tH\x02R\x0b\x64\x65scription\x88\x01\x01\x12\x16\n\x06status\x18\x06 \x01(\tR\x06status\x12\x14\n\x05title\x18\x07 \x01(\tR\x05title:\x02\x18\x01\x42\x0c\n\n_dedup_keyB\r\n\x0b_source_urlB\x0e\n\x0c_description"1\n\x1fTestPagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"<\n\x1eGetPagerDutyIntegrationRequest\x12\x13\n\x02id\x18\x01 \x01(\tH\x00R\x02id\x88\x01\x01\x42\x05\n\x03_id"j\n\x1fGetPagerDutyIntegrationResponse\x12G\n\x0bintegration\x18\x01 \x01(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0bintegration"T\n TestPagerDutyIntegrationResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message"X\n\x1e\x41\x64\x64PagerDutyIntegrationRequest\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x14\n\x05token\x18\x03 \x01(\tR\x05tokenB\x07\n\x05_name"j\n\x1f\x41\x64\x64PagerDutyIntegrationResponse\x12G\n\x0bintegration\x18\x01 \x01(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0bintegration"3\n!DeletePagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"$\n"DeletePagerDutyIntegrationResponse"\xab\x01\n!UpdatePagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1d\n\x07\x64\x65\x66\x61ult\x18\x03 \x01(\x08H\x01R\x07\x64\x65\x66\x61ult\x88\x01\x01\x12\x1f\n\x05token\x18\x04 \x01(\tB\x04\xd8\xa1\'\x01H\x02R\x05token\x88\x01\x01\x42\x07\n\x05_nameB\n\n\x08_defaultB\x08\n\x06_token"m\n"UpdatePagerDutyIntegrationResponse\x12G\n\x0bintegration\x18\x02 \x01(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0bintegration"7\n%SetDefaultPagerDutyIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"8\n&SetDefaultPagerDutyIntegrationResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"$\n"GetAllPagerDutyIntegrationsRequest"p\n#GetAllPagerDutyIntegrationsResponse\x12I\n\x0cintegrations\x18\x01 \x03(\x0b\x32%.chalk.server.v1.PagerDutyIntegrationR\x0cintegrations"2\n TestIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"U\n!TestIncidentIoIntegrationResponse\x12\x16\n\x06status\x18\x01 \x01(\tR\x06status\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message"1\n\x1fGetIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"l\n GetIncidentIoIntegrationResponse\x12H\n\x0bintegration\x18\x01 \x01(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0bintegration"\x93\x01\n\x1f\x41\x64\x64IncidentIoIntegrationRequest\x12.\n\x10integration_name\x18\x01 \x01(\tH\x00R\x0fintegrationName\x88\x01\x01\x12+\n\x11integration_token\x18\x03 \x01(\tR\x10integrationTokenB\x13\n\x11_integration_name"l\n AddIncidentIoIntegrationResponse\x12H\n\x0bintegration\x18\x01 \x01(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0bintegration"4\n"DeleteIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id"%\n#DeleteIncidentIoIntegrationResponse"\x81\x01\n"UpdateIncidentIoIntegrationRequest\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x17\n\x04name\x18\x02 \x01(\tH\x00R\x04name\x88\x01\x01\x12\x1f\n\x05token\x18\x03 \x01(\tB\x04\xd8\xa1\'\x01H\x01R\x05token\x88\x01\x01\x42\x07\n\x05_nameB\x08\n\x06_token"o\n#UpdateIncidentIoIntegrationResponse\x12H\n\x0bintegration\x18\x01 \x01(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0bintegration"%\n#GetAllIncidentIoIntegrationsRequest"r\n$GetAllIncidentIoIntegrationsResponse\x12J\n\x0cintegrations\x18\x01 \x03(\x0b\x32&.chalk.server.v1.IncidentIoIntegrationR\x0cintegrations*\xb8\x01\n\x11PagerDutySeverity\x12#\n\x1fPAGER_DUTY_SEVERITY_UNSPECIFIED\x10\x00\x12\x1c\n\x18PAGER_DUTY_SEVERITY_INFO\x10\x01\x12\x1f\n\x1bPAGER_DUTY_SEVERITY_WARNING\x10\x02\x12\x1d\n\x19PAGER_DUTY_SEVERITY_ERROR\x10\x03\x12 \n\x1cPAGER_DUTY_SEVERITY_CRITICAL\x10\x04*\xb2\x01\n\x14PagerDutyEventAction\x12\'\n#PAGER_DUTY_EVENT_ACTION_UNSPECIFIED\x10\x00\x12#\n\x1fPAGER_DUTY_EVENT_ACTION_TRIGGER\x10\x01\x12\'\n#PAGER_DUTY_EVENT_ACTION_ACKNOWLEDGE\x10\x02\x12#\n\x1fPAGER_DUTY_EVENT_ACTION_RESOLVE\x10\x03\x32\xef\x0e\n\x11MonitoringService\x12\x84\x01\n\x18TestPagerDutyIntegration\x12\x30.chalk.server.v1.TestPagerDutyIntegrationRequest\x1a\x31.chalk.server.v1.TestPagerDutyIntegrationResponse"\x03\x80}\x06\x12\x87\x01\n\x17\x41\x64\x64PagerDutyIntegration\x12/.chalk.server.v1.AddPagerDutyIntegrationRequest\x1a\x30.chalk.server.v1.AddPagerDutyIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x90\x01\n\x1a\x44\x65letePagerDutyIntegration\x12\x32.chalk.server.v1.DeletePagerDutyIntegrationRequest\x1a\x33.chalk.server.v1.DeletePagerDutyIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x9f\x01\n\x1eSetDefaultPagerDutyIntegration\x12\x36.chalk.server.v1.SetDefaultPagerDutyIntegrationRequest\x1a\x37.chalk.server.v1.SetDefaultPagerDutyIntegrationResponse"\x0c\x90\x02\x02\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x93\x01\n\x1aUpdatePagerDutyIntegration\x12\x32.chalk.server.v1.UpdatePagerDutyIntegrationRequest\x1a\x33.chalk.server.v1.UpdatePagerDutyIntegrationResponse"\x0c\x90\x02\x02\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x90\x01\n\x1bGetAllPagerDutyIntegrations\x12\x33.chalk.server.v1.GetAllPagerDutyIntegrationsRequest\x1a\x34.chalk.server.v1.GetAllPagerDutyIntegrationsResponse"\x06\x90\x02\x01\x80}\x06\x12\x84\x01\n\x17GetPagerDutyIntegration\x12/.chalk.server.v1.GetPagerDutyIntegrationRequest\x1a\x30.chalk.server.v1.GetPagerDutyIntegrationResponse"\x06\x90\x02\x01\x80}\x06\x12\x87\x01\n\x19TestIncidentIoIntegration\x12\x31.chalk.server.v1.TestIncidentIoIntegrationRequest\x1a\x32.chalk.server.v1.TestIncidentIoIntegrationResponse"\x03\x80}\x06\x12\x8a\x01\n\x18\x41\x64\x64IncidentIoIntegration\x12\x30.chalk.server.v1.AddIncidentIoIntegrationRequest\x1a\x31.chalk.server.v1.AddIncidentIoIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x93\x01\n\x1b\x44\x65leteIncidentIoIntegration\x12\x33.chalk.server.v1.DeleteIncidentIoIntegrationRequest\x1a\x34.chalk.server.v1.DeleteIncidentIoIntegrationResponse"\t\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x96\x01\n\x1bUpdateIncidentIoIntegration\x12\x33.chalk.server.v1.UpdateIncidentIoIntegrationRequest\x1a\x34.chalk.server.v1.UpdateIncidentIoIntegrationResponse"\x0c\x90\x02\x02\x80}\x05\x8a\xd3\x0e\x02\x08\x02\x12\x93\x01\n\x1cGetAllIncidentIoIntegrations\x12\x34.chalk.server.v1.GetAllIncidentIoIntegrationsRequest\x1a\x35.chalk.server.v1.GetAllIncidentIoIntegrationsResponse"\x06\x90\x02\x01\x80}\x06\x12\x87\x01\n\x18GetIncidentIoIntegration\x12\x30.chalk.server.v1.GetIncidentIoIntegrationRequest\x1a\x31.chalk.server.v1.GetIncidentIoIntegrationResponse"\x06\x90\x02\x01\x80}\x06\x42\x98\x01\n\x13\x63om.chalk.server.v1B\x0fMonitoringProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.monitoring_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\017MonitoringProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_PAGERDUTYINTEGRATION"].fields_by_name["token"]._options = None
    _globals["_PAGERDUTYINTEGRATION"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["token"]._options = None
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_id"]._options = None
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_id"]._serialized_options = b"\030\001"
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_token"]._options = None
    _globals["_INCIDENTIOINTEGRATION"].fields_by_name["source_token"]._serialized_options = b"\030\001"
    _globals["_INCIDENTIOEVENTV2"].fields_by_name["route_token"]._options = None
    _globals["_INCIDENTIOEVENTV2"].fields_by_name["route_token"]._serialized_options = b"\330\241'\001"
    _globals["_INCIDENTIOEVENTV2"]._options = None
    _globals["_INCIDENTIOEVENTV2"]._serialized_options = b"\030\001"
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"].fields_by_name["token"]._options = None
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"].fields_by_name["token"]._options = None
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"].fields_by_name["token"]._serialized_options = b"\330\241'\001"
    _globals["_MONITORINGSERVICE"].methods_by_name["TestPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name["TestPagerDutyIntegration"]._serialized_options = b"\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["AddPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "AddPagerDutyIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["DeletePagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "DeletePagerDutyIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["SetDefaultPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "SetDefaultPagerDutyIntegration"
    ]._serialized_options = b"\220\002\002\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["UpdatePagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "UpdatePagerDutyIntegration"
    ]._serialized_options = b"\220\002\002\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetAllPagerDutyIntegrations"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetAllPagerDutyIntegrations"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetPagerDutyIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetPagerDutyIntegration"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["TestIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name["TestIncidentIoIntegration"]._serialized_options = b"\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["AddIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "AddIncidentIoIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["DeleteIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "DeleteIncidentIoIntegration"
    ]._serialized_options = b"\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["UpdateIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "UpdateIncidentIoIntegration"
    ]._serialized_options = b"\220\002\002\200}\005\212\323\016\002\010\002"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetAllIncidentIoIntegrations"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetAllIncidentIoIntegrations"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_MONITORINGSERVICE"].methods_by_name["GetIncidentIoIntegration"]._options = None
    _globals["_MONITORINGSERVICE"].methods_by_name[
        "GetIncidentIoIntegration"
    ]._serialized_options = b"\220\002\001\200}\006"
    _globals["_PAGERDUTYSEVERITY"]._serialized_start = 4149
    _globals["_PAGERDUTYSEVERITY"]._serialized_end = 4333
    _globals["_PAGERDUTYEVENTACTION"]._serialized_start = 4336
    _globals["_PAGERDUTYEVENTACTION"]._serialized_end = 4514
    _globals["_PAGERDUTYEVENTV2PAYLOAD"]._serialized_start = 213
    _globals["_PAGERDUTYEVENTV2PAYLOAD"]._serialized_end = 552
    _globals["_PAGERDUTYEVENTV2LINK"]._serialized_start = 554
    _globals["_PAGERDUTYEVENTV2LINK"]._serialized_end = 630
    _globals["_PAGERDUTYEVENTV2IMAGE"]._serialized_start = 632
    _globals["_PAGERDUTYEVENTV2IMAGE"]._serialized_end = 738
    _globals["_PAGERDUTYEVENTV2"]._serialized_start = 741
    _globals["_PAGERDUTYEVENTV2"]._serialized_end = 1198
    _globals["_PAGERDUTYINTEGRATION"]._serialized_start = 1201
    _globals["_PAGERDUTYINTEGRATION"]._serialized_end = 1366
    _globals["_INCIDENTIOINTEGRATION"]._serialized_start = 1369
    _globals["_INCIDENTIOINTEGRATION"]._serialized_end = 1653
    _globals["_INCIDENTIOEVENTV2"]._serialized_start = 1656
    _globals["_INCIDENTIOEVENTV2"]._serialized_end = 1945
    _globals["_TESTPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 1947
    _globals["_TESTPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 1996
    _globals["_GETPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 1998
    _globals["_GETPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2058
    _globals["_GETPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2060
    _globals["_GETPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2166
    _globals["_TESTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2168
    _globals["_TESTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2252
    _globals["_ADDPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2254
    _globals["_ADDPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2342
    _globals["_ADDPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2344
    _globals["_ADDPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2450
    _globals["_DELETEPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2452
    _globals["_DELETEPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2503
    _globals["_DELETEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2505
    _globals["_DELETEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2541
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2544
    _globals["_UPDATEPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2715
    _globals["_UPDATEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2717
    _globals["_UPDATEPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2826
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONREQUEST"]._serialized_start = 2828
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONREQUEST"]._serialized_end = 2883
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_start = 2885
    _globals["_SETDEFAULTPAGERDUTYINTEGRATIONRESPONSE"]._serialized_end = 2941
    _globals["_GETALLPAGERDUTYINTEGRATIONSREQUEST"]._serialized_start = 2943
    _globals["_GETALLPAGERDUTYINTEGRATIONSREQUEST"]._serialized_end = 2979
    _globals["_GETALLPAGERDUTYINTEGRATIONSRESPONSE"]._serialized_start = 2981
    _globals["_GETALLPAGERDUTYINTEGRATIONSRESPONSE"]._serialized_end = 3093
    _globals["_TESTINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3095
    _globals["_TESTINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3145
    _globals["_TESTINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3147
    _globals["_TESTINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3232
    _globals["_GETINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3234
    _globals["_GETINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3283
    _globals["_GETINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3285
    _globals["_GETINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3393
    _globals["_ADDINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3396
    _globals["_ADDINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3543
    _globals["_ADDINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3545
    _globals["_ADDINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3653
    _globals["_DELETEINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3655
    _globals["_DELETEINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3707
    _globals["_DELETEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3709
    _globals["_DELETEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3746
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"]._serialized_start = 3749
    _globals["_UPDATEINCIDENTIOINTEGRATIONREQUEST"]._serialized_end = 3878
    _globals["_UPDATEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_start = 3880
    _globals["_UPDATEINCIDENTIOINTEGRATIONRESPONSE"]._serialized_end = 3991
    _globals["_GETALLINCIDENTIOINTEGRATIONSREQUEST"]._serialized_start = 3993
    _globals["_GETALLINCIDENTIOINTEGRATIONSREQUEST"]._serialized_end = 4030
    _globals["_GETALLINCIDENTIOINTEGRATIONSRESPONSE"]._serialized_start = 4032
    _globals["_GETALLINCIDENTIOINTEGRATIONSRESPONSE"]._serialized_end = 4146
    _globals["_MONITORINGSERVICE"]._serialized_start = 4517
    _globals["_MONITORINGSERVICE"]._serialized_end = 6420
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/protosql/v1/sql_service.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.common.v1 import chalk_error_pb2 as chalk_dot_common_dot_v1_dot_chalk__error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n#chalk/protosql/v1/sql_service.proto\x12\x11\x63halk.protosql.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a!chalk/common/v1/chalk_error.proto".\n\x16\x45xecuteSqlQueryRequest\x12\x14\n\x05query\x18\x01 \x01(\tR\x05query"\x90\x01\n\x17\x45xecuteSqlQueryResponse\x12\x19\n\x08query_id\x18\x01 \x01(\tR\x07queryId\x12\x1a\n\x07parquet\x18\x02 \x01(\x0cH\x00R\x07parquet\x12\x33\n\x06\x65rrors\x18\x03 \x03(\x0b\x32\x1b.chalk.common.v1.ChalkErrorR\x06\x65rrorsB\t\n\x07payload"+\n\x13PlanSqlQueryRequest\x12\x14\n\x05query\x18\x01 \x01(\tR\x05query"9\n\x14PlanSqlQueryResponse\x12!\n\x0clogical_plan\x18\x01 \x01(\tR\x0blogicalPlan2\xe1\x01\n\nSqlService\x12m\n\x0f\x45xecuteSqlQuery\x12).chalk.protosql.v1.ExecuteSqlQueryRequest\x1a*.chalk.protosql.v1.ExecuteSqlQueryResponse"\x03\x80}\x03\x12\x64\n\x0cPlanSqlQuery\x12&.chalk.protosql.v1.PlanSqlQueryRequest\x1a\'.chalk.protosql.v1.PlanSqlQueryResponse"\x03\x80}\x03\x42\x8e\x01\n\x15\x63om.chalk.protosql.v1B\x0fSqlServiceProtoP\x01\xa2\x02\x03\x43PX\xaa\x02\x11\x43halk.Protosql.V1\xca\x02\x11\x43halk\\Protosql\\V1\xe2\x02\x1d\x43halk\\Protosql\\V1\\GPBMetadata\xea\x02\x13\x43halk::Protosql::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.protosql.v1.sql_service_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\025com.chalk.protosql.v1B\017SqlServiceProtoP\001\242\002\003CPX\252\002\021Chalk.Protosql.V1\312\002\021Chalk\\Protosql\\V1\342\002\035Chalk\\Protosql\\V1\\GPBMetadata\352\002\023Chalk::Protosql::V1"
    _globals["_SQLSERVICE"].methods_by_name["ExecuteSqlQuery"]._options = None
    _globals["_SQLSERVICE"].methods_by_name["ExecuteSqlQuery"]._serialized_options = b"\200}\003"
    _globals["_SQLSERVICE"].methods_by_name["PlanSqlQuery"]._options = None
    _globals["_SQLSERVICE"].methods_by_name["PlanSqlQuery"]._serialized_options = b"\200}\003"
    _globals["_EXECUTESQLQUERYREQUEST"]._serialized_start = 126
    _globals["_EXECUTESQLQUERYREQUEST"]._serialized_end = 172
    _globals["_EXECUTESQLQUERYRESPONSE"]._serialized_start = 175
    _globals["_EXECUTESQLQUERYRESPONSE"]._serialized_end = 319
    _globals["_PLANSQLQUERYREQUEST"]._serialized_start = 321
    _globals["_PLANSQLQUERYREQUEST"]._serialized_end = 364
    _globals["_PLANSQLQUERYRESPONSE"]._serialized_start = 366
    _globals["_PLANSQLQUERYRESPONSE"]._serialized_end = 423
    _globals["_SQLSERVICE"]._serialized_start = 426
    _globals["_SQLSERVICE"]._serialized_end = 651
# @@protoc_insertion_point(module_scope)



from chalk._gen.chalk.common.v1 import chalk_error_pb2 as _chalk_error_pb2
from chalk._gen.chalk.expression.v1 import expression_pb2 as _expression_pb2
from chalk._gen.chalk.graph.v1 import graph_pb2 as _graph_pb2
from google.protobuf import duration_pb2 as _duration_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class FeatherBodyType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    FEATHER_BODY_TYPE_UNSPECIFIED: _ClassVar[FeatherBodyType]
    FEATHER_BODY_TYPE_TABLE: _ClassVar[FeatherBodyType]
    FEATHER_BODY_TYPE_RECORD_BATCHES: _ClassVar[FeatherBodyType]

FEATHER_BODY_TYPE_UNSPECIFIED: FeatherBodyType
FEATHER_BODY_TYPE_TABLE: FeatherBodyType
FEATHER_BODY_TYPE_RECORD_BATCHES: FeatherBodyType

class OnlineQueryRequest(_message.Message):
    __slots__ = ("inputs", "outputs", "now", "staleness", "context", "response_options")
    class InputsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...
        ) -> None: ...

    class StalenessEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    INPUTS_FIELD_NUMBER: _ClassVar[int]
    OUTPUTS_FIELD_NUMBER: _ClassVar[int]
    NOW_FIELD_NUMBER: _ClassVar[int]
    STALENESS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    inputs: _containers.MessageMap[str, _struct_pb2.Value]
    outputs: _containers.RepeatedCompositeFieldContainer[OutputExpr]
    now: _timestamp_pb2.Timestamp
    staleness: _containers.ScalarMap[str, str]
    context: OnlineQueryContext
    response_options: OnlineQueryResponseOptions
    def __init__(
        self,
        inputs: _Optional[_Mapping[str, _struct_pb2.Value]] = ...,
        outputs: _Optional[_Iterable[_Union[OutputExpr, _Mapping]]] = ...,
        now: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        staleness: _Optional[_Mapping[str, str]] = ...,
        context: _Optional[_Union[OnlineQueryContext, _Mapping]] = ...,
        response_options: _Optional[_Union[OnlineQueryResponseOptions, _Mapping]] = ...,
    ) -> None: ...

class OnlineQueryBulkRequest(_message.Message):
    __slots__ = ("inputs_feather", "outputs", "now", "staleness", "context", "response_options", "body_type")
    class StalenessEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    INPUTS_FEATHER_FIELD_NUMBER: _ClassVar[int]
    OUTPUTS_FIELD_NUMBER: _ClassVar[int]
    NOW_FIELD_NUMBER: _ClassVar[int]
    STALENESS_FIELD_NUMBER: _ClassVar[int]
    CONTEXT_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    BODY_TYPE_FIELD_NUMBER: _ClassVar[int]
    inputs_feather: bytes
    outputs: _containers.RepeatedCompositeFieldContainer[OutputExpr]
    now: _containers.RepeatedCompositeFieldContainer[_timestamp_pb2.Timestamp]
    staleness: _containers.ScalarMap[str, str]
    context: OnlineQueryContext
    response_options: OnlineQueryResponseOptions
    body_type: FeatherBodyType
    def __init__(
        self,
        inputs_feather: _Optional[bytes] = ...,
        outputs: _Optional[_Iterable[_Union[OutputExpr, _Mapping]]] = ...,
        now: _Optional[_Iterable[_Union[_timestamp_pb2.Timestamp, _Mapping]]] = ...,
        staleness: _Optional[_Mapping[str, str]] = ...,
        context: _Optional[_Union[OnlineQueryContext, _Mapping]] = ...,
        response_options: _Optional[_Union[OnlineQueryResponseOptions, _Mapping]] = ...,
        body_type: _Optional[_Union[FeatherBodyType, str]] = ...,
    ) -> None: ...

class GenericSingleQuery(_message.Message):
    __slots__ = ("single_request", "bulk_request")
    SINGLE_REQUEST_FIELD_NUMBER: _ClassVar[int]
    BULK_REQUEST_FIELD_NUMBER: _ClassVar[int]
    single_request: OnlineQueryRequest
    bulk_request: OnlineQueryBulkRequest
    def __init__(
        self,
        single_request: _Optional[_Union[OnlineQueryRequest, _Mapping]] = ...,
        bulk_request: _Optional[_Union[OnlineQueryBulkRequest, _Mapping]] = ...,
    ) -> None: ...

class OnlineQueryMultiRequest(_message.Message):
    __slots__ = ("queries",)
    QUERIES_FIELD_NUMBER: _ClassVar[int]
    queries: _containers.RepeatedCompositeFieldContainer[GenericSingleQuery]
    def __init__(self, queries: _Optional[_Iterable[_Union[GenericSingleQuery, _Mapping]]] = ...) -> None: ...

class FeatureExpression(_message.Message):
    __slots__ = ("output_column_name", "namespace", "expr")
    OUTPUT_COLUMN_NAME_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    EXPR_FIELD_NUMBER: _ClassVar[int]
    output_column_name: str
    namespace: str
    expr: _expression_pb2.LogicalExprNode
    def __init__(
        self,
        output_column_name: _Optional[str] = ...,
        namespace: _Optional[str] = ...,
        expr: _Optional[_Union[_expression_pb2.LogicalExprNode, _Mapping]] = ...,
    ) -> None: ...

class OutputExpr(_message.Message):
    __slots__ = ("feature_fqn", "feature_expression")
    FEATURE_FQN_FIELD_NUMBER: _ClassVar[int]
    FEATURE_EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    feature_fqn: str
    feature_expression: FeatureExpression
    def __init__(
        self,
        feature_fqn: _Optional[str] = ...,
        feature_expression: _Optional[_Union[FeatureExpression, _Mapping]] = ...,
    ) -> None: ...

class OnlineQueryContext(_message.Message):
    __slots__ = (
        "environment",
        "tags",
        "required_resolver_tags",
        "deployment_id",
        "branch_id",
        "correlation_id",
        "query_name",
        "query_name_version",
        "options",
        "value_metrics_tag_by_features",
        "query_context",
        "overlay_graph",
    )
    class OptionsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...
        ) -> None: ...

    class QueryContextEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...
        ) -> None: ...

    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    REQUIRED_RESOLVER_TAGS_FIELD_NUMBER: _ClassVar[int]
    DEPLOYMENT_ID_FIELD_NUMBER: _ClassVar[int]
    BRANCH_ID_FIELD_NUMBER: _ClassVar[int]
    CORRELATION_ID_FIELD_NUMBER: _ClassVar[int]
    QUERY_NAME_FIELD_NUMBER: _ClassVar[int]
    QUERY_NAME_VERSION_FIELD_NUMBER: _ClassVar[int]
    OPTIONS_FIELD_NUMBER: _ClassVar[int]
    VALUE_METRICS_TAG_BY_FEATURES_FIELD_NUMBER: _ClassVar[int]
    QUERY_CONTEXT_FIELD_NUMBER: _ClassVar[int]
    OVERLAY_GRAPH_FIELD_NUMBER: _ClassVar[int]
    environment: str
    tags: _containers.RepeatedScalarFieldContainer[str]
    required_resolver_tags: _containers.RepeatedScalarFieldContainer[str]
    deployment_id: str
    branch_id: str
    correlation_id: str
    query_name: str
    query_name_version: str
    options: _containers.MessageMap[str, _struct_pb2.Value]
    value_metrics_tag_by_features: _containers.RepeatedCompositeFieldContainer[OutputExpr]
    query_context: _containers.MessageMap[str, _struct_pb2.Value]
    overlay_graph: _graph_pb2.OverlayGraph
    def __init__(
        self,
        environment: _Optional[str] = ...,
        tags: _Optional[_Iterable[str]] = ...,
        required_resolver_tags: _Optional[_Iterable[str]] = ...,
        deployment_id: _Optional[str] = ...,
        branch_id: _Optional[str] = ...,
        correlation_id: _Optional[str] = ...,
        query_name: _Optional[str] = ...,
        query_name_version: _Optional[str] = ...,
        options: _Optional[_Mapping[str, _struct_pb2.Value]] = ...,
        value_metrics_tag_by_features: _Optional[_Iterable[_Union[OutputExpr, _Mapping]]] = ...,
        query_context: _Optional[_Mapping[str, _struct_pb2.Value]] = ...,
        overlay_graph: _Optional[_Union[_graph_pb2.OverlayGraph, _Mapping]] = ...,
    ) -> None: ...

class OnlineQueryResponseOptions(_message.Message):
    __slots__ = ("include_meta", "explain", "encoding_options", "metadata")
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    INCLUDE_META_FIELD_NUMBER: _ClassVar[int]
    EXPLAIN_FIELD_NUMBER: _ClassVar[int]
    ENCODING_OPTIONS_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    include_meta: bool
    explain: ExplainOptions
    encoding_options: FeatureEncodingOptions
    metadata: _containers.ScalarMap[str, str]
    def __init__(
        self,
        include_meta: bool = ...,
        explain: _Optional[_Union[ExplainOptions, _Mapping]] = ...,
        encoding_options: _Optional[_Union[FeatureEncodingOptions, _Mapping]] = ...,
        metadata: _Optional[_Mapping[str, str]] = ...,
    ) -> None: ...

class ExplainOptions(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class FeatureEncodingOptions(_message.Message):
    __slots__ = ("encode_structs_as_objects",)
    ENCODE_STRUCTS_AS_OBJECTS_FIELD_NUMBER: _ClassVar[int]
    encode_structs_as_objects: bool
    def __init__(self, encode_structs_as_objects: bool = ...) -> None: ...

class OnlineQueryResponse(_message.Message):
    __slots__ = ("data", "errors", "response_meta")
    DATA_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_META_FIELD_NUMBER: _ClassVar[int]
    data: OnlineQueryResult
    errors: _containers.RepeatedCompositeFieldContainer[_chalk_error_pb2.ChalkError]
    response_meta: OnlineQueryMetadata
    def __init__(
        self,
        data: _Optional[_Union[OnlineQueryResult, _Mapping]] = ...,
        errors: _Optional[_Iterable[_Union[_chalk_error_pb2.ChalkError, _Mapping]]] = ...,
        response_meta: _Optional[_Union[OnlineQueryMetadata, _Mapping]] = ...,
    ) -> None: ...

class OnlineQueryBulkResponse(_message.Message):
    __slots__ = ("scalars_data", "groups_data", "errors", "response_meta")
    class GroupsDataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: bytes
        def __init__(self, key: _Optional[str] = ..., value: _Optional[bytes] = ...) -> None: ...

    SCALARS_DATA_FIELD_NUMBER: _ClassVar[int]
    GROUPS_DATA_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    RESPONSE_META_FIELD_NUMBER: _ClassVar[int]
    scalars_data: bytes
    groups_data: _containers.ScalarMap[str, bytes]
    errors: _containers.RepeatedCompositeFieldContainer[_chalk_error_pb2.ChalkError]
    response_meta: OnlineQueryMetadata
    def __init__(
        self,
        scalars_data: _Optional[bytes] = ...,
        groups_data: _Optional[_Mapping[str, bytes]] = ...,
        errors: _Optional[_Iterable[_Union[_chalk_error_pb2.ChalkError, _Mapping]]] = ...,
        response_meta: _Optional[_Union[OnlineQueryMetadata, _Mapping]] = ...,
    ) -> None: ...

class GenericSingleResponse(_message.Message):
    __slots__ = ("single_response", "bulk_response")
    SINGLE_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    BULK_RESPONSE_FIELD_NUMBER: _ClassVar[int]
    single_response: OnlineQueryResponse
    bulk_response: OnlineQueryBulkResponse
    def __init__(
        self,
        single_response: _Optional[_Union[OnlineQueryResponse, _Mapping]] = ...,
        bulk_response: _Optional[_Union[OnlineQueryBulkResponse, _Mapping]] = ...,
    ) -> None: ...

class OnlineQueryMultiResponse(_message.Message):
    __slots__ = ("responses", "errors")
    RESPONSES_FIELD_NUMBER: _ClassVar[int]
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    responses: _containers.RepeatedCompositeFieldContainer[GenericSingleResponse]
    errors: _containers.RepeatedCompositeFieldContainer[_chalk_error_pb2.ChalkError]
    def __init__(
        self,
        responses: _Optional[_Iterable[_Union[GenericSingleResponse, _Mapping]]] = ...,
        errors: _Optional[_Iterable[_Union[_chalk_error_pb2.ChalkError, _Mapping]]] = ...,
    ) -> None: ...

class OnlineQueryResult(_message.Message):
    __slots__ = ("results",)
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[FeatureResult]
    def __init__(self, results: _Optional[_Iterable[_Union[FeatureResult, _Mapping]]] = ...) -> None: ...

class FeatureResult(_message.Message):
    __slots__ = ("field", "pkey", "value", "error", "ts", "meta")
    FIELD_FIELD_NUMBER: _ClassVar[int]
    PKEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    TS_FIELD_NUMBER: _ClassVar[int]
    META_FIELD_NUMBER: _ClassVar[int]
    field: str
    pkey: _struct_pb2.Value
    value: _struct_pb2.Value
    error: _chalk_error_pb2.ChalkError
    ts: _timestamp_pb2.Timestamp
    meta: FeatureMeta
    def __init__(
        self,
        field: _Optional[str] = ...,
        pkey: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...,
        value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...,
        error: _Optional[_Union[_chalk_error_pb2.ChalkError, _Mapping]] = ...,
        ts: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        meta: _Optional[_Union[FeatureMeta, _Mapping]] = ...,
    ) -> None: ...

class FeatureMeta(_message.Message):
    __slots__ = ("chosen_resolver_fqn", "cache_hit", "primitive_type", "version")
    CHOSEN_RESOLVER_FQN_FIELD_NUMBER: _ClassVar[int]
    CACHE_HIT_FIELD_NUMBER: _ClassVar[int]
    PRIMITIVE_TYPE_FIELD_NUMBER: _ClassVar[int]
    VERSION_FIELD_NUMBER: _ClassVar[int]
    chosen_resolver_fqn: str
    cache_hit: bool
    primitive_type: str
    version: int
    def __init__(
        self,
        chosen_resolver_fqn: _Optional[str] = ...,
        cache_hit: bool = ...,
        primitive_type: _Optional[str] = ...,
        version: _Optional[int] = ...,
    ) -> None: ...

class OnlineQueryMetadata(_message.Message):
    __slots__ = (
        "execution_duration",
        "deployment_id",
        "environment_id",
        "environment_name",
        "query_id",
        "query_timestamp",
        "query_hash",
        "explain_output",
        "metadata",
        "additional_metadata",
    )
    class MetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    class AdditionalMetadataEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: _struct_pb2.Value
        def __init__(
            self, key: _Optional[str] = ..., value: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...
        ) -> None: ...

    EXECUTION_DURATION_FIELD_NUMBER: _ClassVar[int]
    DEPLOYMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_NAME_FIELD_NUMBER: _ClassVar[int]
    QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    QUERY_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    QUERY_HASH_FIELD_NUMBER: _ClassVar[int]
    EXPLAIN_OUTPUT_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    ADDITIONAL_METADATA_FIELD_NUMBER: _ClassVar[int]
    execution_duration: _duration_pb2.Duration
    deployment_id: str
    environment_id: str
    environment_name: str
    query_id: str
    query_timestamp: _timestamp_pb2.Timestamp
    query_hash: str
    explain_output: QueryExplainInfo
    metadata: _containers.ScalarMap[str, str]
    additional_metadata: _containers.MessageMap[str, _struct_pb2.Value]
    def __init__(
        self,
        execution_duration: _Optional[_Union[_duration_pb2.Duration, _Mapping]] = ...,
        deployment_id: _Optional[str] = ...,
        environment_id: _Optional[str] = ...,
        environment_name: _Optional[str] = ...,
        query_id: _Optional[str] = ...,
        query_timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        query_hash: _Optional[str] = ...,
        explain_output: _Optional[_Union[QueryExplainInfo, _Mapping]] = ...,
        metadata: _Optional[_Mapping[str, str]] = ...,
        additional_metadata: _Optional[_Mapping[str, _struct_pb2.Value]] = ...,
    ) -> None: ...

class QueryExplainInfo(_message.Message):
    __slots__ = ("plan_string",)
    PLAN_STRING_FIELD_NUMBER: _ClassVar[int]
    plan_string: str
    def __init__(self, plan_string: _Optional[str] = ...) -> None: ...

class UploadFeaturesBulkRequest(_message.Message):
    __slots__ = ("inputs_feather", "body_type")
    INPUTS_FEATHER_FIELD_NUMBER: _ClassVar[int]
    BODY_TYPE_FIELD_NUMBER: _ClassVar[int]
    inputs_feather: bytes
    body_type: FeatherBodyType
    def __init__(
        self, inputs_feather: _Optional[bytes] = ..., body_type: _Optional[_Union[FeatherBodyType, str]] = ...
    ) -> None: ...

class UploadFeaturesBulkResponse(_message.Message):
    __slots__ = ("errors",)
    ERRORS_FIELD_NUMBER: _ClassVar[int]
    errors: _containers.RepeatedCompositeFieldContainer[_chalk_error_pb2.ChalkError]
    def __init__(self, errors: _Optional[_Iterable[_Union[_chalk_error_pb2.ChalkError, _Mapping]]] = ...) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

from abc import (
    ABCMeta,
    abstractmethod,
)
from chalk._gen.chalk.server.v1.offline_queries_pb2 import (
    GetOfflineQueryRequest,
    GetOfflineQueryResponse,
    ListOfflineQueriesRequest,
    ListOfflineQueriesResponse,
)
from chalk._gen.chalk.server.v1.performance_summary_pb2 import (
    ListOfflineQueryShardPerformanceSummariesRequest,
    ListOfflineQueryShardPerformanceSummariesResponse,
)
from grpc import (
    Channel,
    Server,
    ServicerContext,
    UnaryUnaryMultiCallable,
)

class OfflineQueryMetadataServiceStub:
    """message GetOfflineQueryShardsRequest {
     string offline_query_id = 1;
    }

    message GetOfflineQueryShardsResponse {
     repeated OfflineQueryShard offline_query_shards = 1;
    }
    """

    def __init__(self, channel: Channel) -> None: ...
    ListOfflineQueries: UnaryUnaryMultiCallable[
        ListOfflineQueriesRequest,
        ListOfflineQueriesResponse,
    ]
    GetOfflineQuery: UnaryUnaryMultiCallable[
        GetOfflineQueryRequest,
        GetOfflineQueryResponse,
    ]
    ListOfflineQueryShardPerformanceSummaries: UnaryUnaryMultiCallable[
        ListOfflineQueryShardPerformanceSummariesRequest,
        ListOfflineQueryShardPerformanceSummariesResponse,
    ]
    """ rpc GetOfflineQueryShards(GetOfflineQueryShardsRequest) returns (GetOfflineQueryShardsResponse) {
       option(chalk.auth.v1.permission) = PERMISSION_DEPLOY_READ;
       option idempotency_level = NO_SIDE_EFFECTS;
     }
    """

class OfflineQueryMetadataServiceServicer(metaclass=ABCMeta):
    """message GetOfflineQueryShardsRequest {
     string offline_query_id = 1;
    }

    message GetOfflineQueryShardsResponse {
     repeated OfflineQueryShard offline_query_shards = 1;
    }
    """

    @abstractmethod
    def ListOfflineQueries(
        self,
        request: ListOfflineQueriesRequest,
        context: ServicerContext,
    ) -> ListOfflineQueriesResponse: ...
    @abstractmethod
    def GetOfflineQuery(
        self,
        request: GetOfflineQueryRequest,
        context: ServicerContext,
    ) -> GetOfflineQueryResponse: ...
    @abstractmethod
    def ListOfflineQueryShardPerformanceSummaries(
        self,
        request: ListOfflineQueryShardPerformanceSummariesRequest,
        context: ServicerContext,
    ) -> ListOfflineQueryShardPerformanceSummariesResponse:
        """rpc GetOfflineQueryShards(GetOfflineQueryShardsRequest) returns (GetOfflineQueryShardsResponse) {
          option(chalk.auth.v1.permission) = PERMISSION_DEPLOY_READ;
          option idempotency_level = NO_SIDE_EFFECTS;
        }
        """

def add_OfflineQueryMetadataServiceServicer_to_server(
    servicer: OfflineQueryMetadataServiceServicer, server: Server
) -> None: ...



from __future__ import annotations

import dataclasses
import traceback
from typing import Callable, Optional, Sequence, Union

import pyarrow

from chalk import DataFrame, Features, StaticOperator
from chalk._gen.chalk.expression.v1 import expression_pb2 as expr_pb
from chalk.client import ChalkError, ChalkException, ErrorCode, ErrorCodeCategory
from chalk.features.feature_field import Feature


class _GetStaticOperatorError(Exception):
    underlying_error: ChalkError

    def __init__(self, resolver_fqn: str, message: str, underlying_exception: Optional[Exception]):
        super().__init__(f"Failed to get a static operator from the resolver '{resolver_fqn}': {message}")

        def get_stacktrace(exc: Exception):
            try:
                return "".join(traceback.format_exception(exc))
            except TypeError:
                return "".join(traceback.format_exception(type(exc), exc, None))

        self.underlying_error = ChalkError.create(
            code=ErrorCode.RESOLVER_FAILED if underlying_exception else ErrorCode.VALIDATION_FAILED,
            category=ErrorCodeCategory.REQUEST,
            message=message,
            resolver=resolver_fqn,
            exception=ChalkException.create(
                kind=type(underlying_exception).__name__,
                message=str(underlying_exception),
                stacktrace=get_stacktrace(underlying_exception),
            )
            if underlying_exception
            else None,
        )


@dataclasses.dataclass
class DfPlaceholder:
    schema_dict: dict[str, pyarrow.DataType]

    def _to_proto(self) -> expr_pb.LogicalExprNode:
        return expr_pb.LogicalExprNode(
            call=expr_pb.ExprCall(
                func=expr_pb.LogicalExprNode(identifier=expr_pb.Identifier(name="static_df_placeholder")),
                args=[],
                kwargs={},
            )
        )


@dataclasses.dataclass
class ChalkDataFrame:
    def _to_proto(self) -> expr_pb.LogicalExprNode:
        return expr_pb.LogicalExprNode(
            call=expr_pb.ExprCall(
                func=expr_pb.LogicalExprNode(identifier=expr_pb.Identifier(name="chalk_data_frame")),
                args=[],
                kwargs={},
            )
        )


def schema_for_input(input_type: Union[Feature, type[DataFrame]]) -> dict[str, pyarrow.DataType]:
    if isinstance(input_type, Feature):
        return {input_type.name: input_type.converter.pyarrow_dtype}
    elif issubclass(input_type, DataFrame):  # pyright: ignore [reportUnnecessaryIsInstance]
        return {feat.root_fqn: feat.converter.pyarrow_dtype for feat in input_type.columns}
    else:
        raise ValueError(f"Unexpected input type: {input_type}")


def static_resolver_to_operator(
    fqn: str,
    fn: Callable,
    inputs: Sequence[Union[Feature, type[DataFrame]]],
    output: Optional[type[Features]],
) -> StaticOperator | DfPlaceholder | ChalkDataFrame:
    if output is None:
        raise _GetStaticOperatorError(
            resolver_fqn=fqn,
            message="Static resolver must specify a return type",
            underlying_exception=None,
        )

    # TODO CHA-5893: Re-enable this check and permit DF -> DF and () -> DF
    if not (
        len(output.features) == 1 and isinstance(output.features[0], type) and issubclass(output.features[0], DataFrame)
    ):
        raise _GetStaticOperatorError(
            resolver_fqn=fqn,
            message="Static resolver must take no arguments and have exactly one DataFrame output",
            underlying_exception=None,
        )
    try:
        placeholder_inputs = [DfPlaceholder(schema_dict=schema_for_input(input_type)) for input_type in inputs]
        static_operator = fn(*placeholder_inputs)
    except Exception as e:
        # Weird hacky way to return a placeholder even if the resolver fails.
        if len(inputs) > 0:
            return DfPlaceholder(schema_dict={})
        raise _GetStaticOperatorError(
            resolver_fqn=fqn, message="Resolver failed with an exception", underlying_exception=e
        )
    else:
        if (
            not isinstance(static_operator, (StaticOperator, DfPlaceholder))
            and not static_operator.__class__.__name__ == "ChalkDataFrame"
        ):
            raise _GetStaticOperatorError(
                resolver_fqn=fqn,
                message=f"Static resolver must return a StaticOperator, found {type(static_operator).__name__}",
                underlying_exception=None,
            )
        return static_operator



import enum
import functools
from typing import Type


@functools.lru_cache(None)  # Cache all calls to this function
def get_enum_value_type(enum_cls: Type[enum.Enum]) -> type:
    """Get the type of the enum value.
    Requires that all enum member values have the same type;
    otherwise raises a TypeError.
    Returns `None` if the enum has no members.

    Note: if an enum contains both float and int members, then this function will return `float`.

    Parameters
    ----------
    enum_cls
        The enum class

    Returns
    -------
    type
        The type of the enum member values.

    Raises
    ------
    ValueError
        If `enum_cls` has no members.
    TypeError
        If `enum_cls` has members of heterogeneous types, then a TypeError is raised.
    """
    typ = None
    for x in enum_cls:
        if typ is None:
            typ = type(x.value)
            continue
        if issubclass(type(x.value), typ):
            continue
        if issubclass(type(x.value), int) and issubclass(typ, float):
            # Effectively treat ints as a subclass of floats
            continue
        if issubclass(typ, int) and issubclass(type(x.value), float):
            # Same as above, but we encountered the int first
            typ = type(x.value)
            continue
        if issubclass(typ, type(x.value)):
            # One is a subclass of the other. In this case, swap it
            typ = type(x.value)
            continue
        raise TypeError(
            f"Enum {enum_cls.__name__} has members of heterogeneous types ({typ.__name__} and {type(x.value).__name__}); all members must have the same type."
        )
    if typ is None:
        raise ValueError(f"Enum {enum_cls.__name__} has no members")
    return typ



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/link.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.utils.v1 import sensitive_pb2 as chalk_dot_utils_dot_v1_dot_sensitive__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1a\x63halk/server/v1/link.proto\x12\x0f\x63halk.server.v1\x1a\x1e\x63halk/utils/v1/sensitive.proto\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto"\x8e\x02\n\tLinkToken\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x1b\n\tclient_id\x18\x02 \x01(\tR\x08\x63lientId\x12)\n\rclient_secret\x18\x03 \x01(\tB\x04\xd8\xa1\'\x01R\x0c\x63lientSecret\x12\x1d\n\napi_server\x18\x04 \x01(\tR\tapiServer\x12\x32\n\x12\x61\x63tive_environment\x18\x05 \x01(\tH\x00R\x11\x61\x63tiveEnvironment\x88\x01\x01\x12;\n\x0bvalid_until\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\nvalidUntilB\x15\n\x13_active_environment"]\n\x15GetLinkSessionRequest\x12!\n\tlink_code\x18\x01 \x01(\tB\x04\xd8\xa1\'\x01R\x08linkCode\x12!\n\x0cproject_name\x18\x02 \x01(\tR\x0bprojectName"\xce\x01\n\x16GetLinkSessionResponse\x12:\n\x06status\x18\x01 \x01(\x0e\x32".chalk.server.v1.LinkSessionStatusR\x06status\x12\x18\n\x07message\x18\x02 \x01(\tR\x07message\x12\x35\n\x05token\x18\x03 \x01(\x0b\x32\x1a.chalk.server.v1.LinkTokenH\x00R\x05token\x88\x01\x01\x12\x1d\n\nsession_id\x18\x04 \x01(\tR\tsessionIdB\x08\n\x06_token"\x1a\n\x18\x43reateLinkSessionRequest"\xbb\x01\n\x19\x43reateLinkSessionResponse\x12!\n\tlink_code\x18\x01 \x01(\tB\x04\xd8\xa1\'\x01R\x08linkCode\x12!\n\tauth_link\x18\x02 \x01(\tB\x04\xd8\xa1\'\x01R\x08\x61uthLink\x12\x39\n\nexpires_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\texpiresAt\x12\x1d\n\nsession_id\x18\x04 \x01(\tR\tsessionId*\xe0\x01\n\x11LinkSessionStatus\x12#\n\x1fLINK_SESSION_STATUS_UNSPECIFIED\x10\x00\x12\x1f\n\x1bLINK_SESSION_STATUS_PENDING\x10\x01\x12\x1f\n\x1bLINK_SESSION_STATUS_SUCCESS\x10\x02\x12\x1e\n\x1aLINK_SESSION_STATUS_FAILED\x10\x03\x12!\n\x1dLINK_SESSION_STATUS_NOT_FOUND\x10\x04\x12!\n\x1dLINK_SESSION_STATUS_FORBIDDEN\x10\x05\x42\x92\x01\n\x13\x63om.chalk.server.v1B\tLinkProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.link_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\tLinkProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_LINKTOKEN"].fields_by_name["client_secret"]._options = None
    _globals["_LINKTOKEN"].fields_by_name["client_secret"]._serialized_options = b"\330\241'\001"
    _globals["_GETLINKSESSIONREQUEST"].fields_by_name["link_code"]._options = None
    _globals["_GETLINKSESSIONREQUEST"].fields_by_name["link_code"]._serialized_options = b"\330\241'\001"
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["link_code"]._options = None
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["link_code"]._serialized_options = b"\330\241'\001"
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["auth_link"]._options = None
    _globals["_CREATELINKSESSIONRESPONSE"].fields_by_name["auth_link"]._serialized_options = b"\330\241'\001"
    _globals["_LINKSESSIONSTATUS"]._serialized_start = 942
    _globals["_LINKSESSIONSTATUS"]._serialized_end = 1166
    _globals["_LINKTOKEN"]._serialized_start = 147
    _globals["_LINKTOKEN"]._serialized_end = 417
    _globals["_GETLINKSESSIONREQUEST"]._serialized_start = 419
    _globals["_GETLINKSESSIONREQUEST"]._serialized_end = 512
    _globals["_GETLINKSESSIONRESPONSE"]._serialized_start = 515
    _globals["_GETLINKSESSIONRESPONSE"]._serialized_end = 721
    _globals["_CREATELINKSESSIONREQUEST"]._serialized_start = 723
    _globals["_CREATELINKSESSIONREQUEST"]._serialized_end = 749
    _globals["_CREATELINKSESSIONRESPONSE"]._serialized_start = 752
    _globals["_CREATELINKSESSIONRESPONSE"]._serialized_end = 939
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/graph.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.arrow.v1 import arrow_pb2 as chalk_dot_arrow_dot_v1_dot_arrow__pb2
from chalk._gen.chalk.artifacts.v1 import export_pb2 as chalk_dot_artifacts_dot_v1_dot_export__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.common.v1 import chalk_error_pb2 as chalk_dot_common_dot_v1_dot_chalk__error__pb2
from chalk._gen.chalk.graph.v1 import graph_pb2 as chalk_dot_graph_dot_v1_dot_graph__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1b\x63halk/server/v1/graph.proto\x12\x0f\x63halk.server.v1\x1a\x1a\x63halk/arrow/v1/arrow.proto\x1a\x1f\x63halk/artifacts/v1/export.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a!chalk/common/v1/chalk_error.proto\x1a\x1a\x63halk/graph/v1/graph.proto"\xd8\x04\n\nFeatureSQL\x12\x0e\n\x02id\x18\x01 \x01(\x03R\x02id\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12(\n\rdeployment_id\x18\x03 \x01(\tH\x00R\x0c\x64\x65ploymentId\x88\x01\x01\x12\x10\n\x03\x66qn\x18\x04 \x01(\tR\x03\x66qn\x12\x12\n\x04name\x18\x05 \x01(\tR\x04name\x12\x1c\n\tnamespace\x18\x06 \x01(\tR\tnamespace\x12(\n\rmax_staleness\x18\x07 \x01(\tH\x01R\x0cmaxStaleness\x88\x01\x01\x12\x31\n\x15\x65tl_offline_to_online\x18\x08 \x01(\x08R\x12\x65tlOfflineToOnline\x12%\n\x0b\x64\x65scription\x18\t \x01(\tH\x02R\x0b\x64\x65scription\x88\x01\x01\x12\x19\n\x05owner\x18\n \x01(\tH\x03R\x05owner\x88\x01\x01\x12\x12\n\x04tags\x18\x0b \x03(\tR\x04tags\x12\x1b\n\tkind_enum\x18\x0c \x01(\tR\x08kindEnum\x12\x12\n\x04kind\x18\r \x01(\tR\x04kind\x12\x1b\n\twas_reset\x18\x0e \x01(\x08R\x08wasReset\x12.\n\x10internal_version\x18\x0f \x01(\x03H\x04R\x0finternalVersion\x88\x01\x01\x12!\n\x0cis_singleton\x18\x10 \x01(\x08R\x0bisSingletonB\x10\n\x0e_deployment_idB\x10\n\x0e_max_stalenessB\x0e\n\x0c_descriptionB\x08\n\x06_ownerB\x13\n\x11_internal_version"P\n\x15GetFeatureSQLResponse\x12\x37\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32\x1b.chalk.server.v1.FeatureSQLR\x08\x66\x65\x61tures";\n\x14GetFeatureSQLRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId"\x86\x03\n\x0f\x46\x65\x61tureMetadata\x12\x10\n\x03\x66qn\x18\x01 \x01(\tR\x03\x66qn\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12%\n\x0b\x64\x65scription\x18\x04 \x01(\tH\x00R\x0b\x64\x65scription\x88\x01\x01\x12\x19\n\x05owner\x18\x05 \x01(\tH\x01R\x05owner\x88\x01\x01\x12\x12\n\x04tags\x18\x06 \x03(\tR\x04tags\x12(\n\rmax_staleness\x18\x07 \x01(\tH\x02R\x0cmaxStaleness\x88\x01\x01\x12\x31\n\x15\x65tl_offline_to_online\x18\x08 \x01(\x08R\x12\x65tlOfflineToOnline\x12\x34\n\x08pa_dtype\x18\t \x01(\x0b\x32\x19.chalk.arrow.v1.ArrowTypeR\x07paDtype\x12\x1a\n\x08nullable\x18\n \x01(\x08R\x08nullableB\x0e\n\x0c_descriptionB\x08\n\x06_ownerB\x10\n\x0e_max_staleness"\xa7\x01\n\x1bGetFeaturesMetadataResponse\x12<\n\x08\x66\x65\x61tures\x18\x01 \x03(\x0b\x32 .chalk.server.v1.FeatureMetadataR\x08\x66\x65\x61tures\x12%\n\x0e\x65nvironment_id\x18\x02 \x01(\tR\renvironmentId\x12#\n\rdeployment_id\x18\x03 \x01(\tR\x0c\x64\x65ploymentId"=\n\x1aGetFeaturesMetadataRequest\x12\x1f\n\x0b\x66qns_filter\x18\x01 \x03(\tR\nfqnsFilter"\xe6\x01\n\x12UpdateGraphRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId\x12/\n\x05graph\x18\x02 \x01(\x0b\x32\x15.chalk.graph.v1.GraphB\x02\x18\x01R\x05graph\x12\'\n\x0f\x63halkpy_version\x18\x03 \x01(\tR\x0e\x63halkpyVersion\x12\x15\n\x03tag\x18\x04 \x01(\tH\x00R\x03tag\x88\x01\x01\x12\x32\n\x06\x65xport\x18\x05 \x01(\x0b\x32\x1a.chalk.artifacts.v1.ExportR\x06\x65xportB\x06\n\x04_tag"\x15\n\x13UpdateGraphResponse"6\n\x0fGetGraphRequest\x12#\n\rdeployment_id\x18\x01 \x01(\tR\x0c\x64\x65ploymentId"\xe4\x01\n\x10GetGraphResponse\x12/\n\x05graph\x18\x01 \x01(\x0b\x32\x15.chalk.graph.v1.GraphB\x02\x18\x01R\x05graph\x12\'\n\x0f\x63halkpy_version\x18\x02 \x01(\tR\x0e\x63halkpyVersion\x12\x15\n\x03tag\x18\x03 \x01(\tH\x00R\x03tag\x88\x01\x01\x12\x32\n\x06\x65xport\x18\x04 \x01(\x0b\x32\x1a.chalk.artifacts.v1.ExportR\x06\x65xport\x12#\n\rdeployment_id\x18\x05 \x01(\tR\x0c\x64\x65ploymentIdB\x06\n\x04_tag"Q\n\rPythonVersion\x12\x14\n\x05major\x18\x01 \x01(\x03R\x05major\x12\x14\n\x05minor\x18\x02 \x01(\x03R\x05minor\x12\x14\n\x05patch\x18\x03 \x01(\x03R\x05patch"\xe7\x01\n"GetCodegenFeaturesFromGraphRequest\x12(\n\rdeployment_id\x18\x01 \x01(\tH\x00R\x0c\x64\x65ploymentId\x88\x01\x01\x12\x1b\n\x06\x62ranch\x18\x02 \x01(\tH\x01R\x06\x62ranch\x88\x01\x01\x12J\n\x0epython_version\x18\x03 \x01(\x0b\x32\x1e.chalk.server.v1.PythonVersionH\x02R\rpythonVersion\x88\x01\x01\x42\x10\n\x0e_deployment_idB\t\n\x07_branchB\x11\n\x0f_python_version"t\n#GetCodegenFeaturesFromGraphResponse\x12\x18\n\x07\x63odegen\x18\x01 \x01(\tR\x07\x63odegen\x12\x33\n\x06\x65rrors\x18\x02 \x03(\x0b\x32\x1b.chalk.common.v1.ChalkErrorR\x06\x65rrors2\xbb\x04\n\x0cGraphService\x12\x66\n\rGetFeatureSQL\x12%.chalk.server.v1.GetFeatureSQLRequest\x1a&.chalk.server.v1.GetFeatureSQLResponse"\x06\x90\x02\x01\x80}\x0b\x12x\n\x13GetFeaturesMetadata\x12+.chalk.server.v1.GetFeaturesMetadataRequest\x1a,.chalk.server.v1.GetFeaturesMetadataResponse"\x06\x90\x02\x01\x80}\x0b\x12W\n\x08GetGraph\x12 .chalk.server.v1.GetGraphRequest\x1a!.chalk.server.v1.GetGraphResponse"\x06\x90\x02\x01\x80}\x0b\x12]\n\x0bUpdateGraph\x12#.chalk.server.v1.UpdateGraphRequest\x1a$.chalk.server.v1.UpdateGraphResponse"\x03\x80}\x0b\x12\x90\x01\n\x1bGetCodegenFeaturesFromGraph\x12\x33.chalk.server.v1.GetCodegenFeaturesFromGraphRequest\x1a\x34.chalk.server.v1.GetCodegenFeaturesFromGraphResponse"\x06\x90\x02\x01\x80}\x0b\x42\x93\x01\n\x13\x63om.chalk.server.v1B\nGraphProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.graph_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\nGraphProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_UPDATEGRAPHREQUEST"].fields_by_name["graph"]._options = None
    _globals["_UPDATEGRAPHREQUEST"].fields_by_name["graph"]._serialized_options = b"\030\001"
    _globals["_GETGRAPHRESPONSE"].fields_by_name["graph"]._options = None
    _globals["_GETGRAPHRESPONSE"].fields_by_name["graph"]._serialized_options = b"\030\001"
    _globals["_GRAPHSERVICE"].methods_by_name["GetFeatureSQL"]._options = None
    _globals["_GRAPHSERVICE"].methods_by_name["GetFeatureSQL"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_GRAPHSERVICE"].methods_by_name["GetFeaturesMetadata"]._options = None
    _globals["_GRAPHSERVICE"].methods_by_name["GetFeaturesMetadata"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_GRAPHSERVICE"].methods_by_name["GetGraph"]._options = None
    _globals["_GRAPHSERVICE"].methods_by_name["GetGraph"]._serialized_options = b"\220\002\001\200}\013"
    _globals["_GRAPHSERVICE"].methods_by_name["UpdateGraph"]._options = None
    _globals["_GRAPHSERVICE"].methods_by_name["UpdateGraph"]._serialized_options = b"\200}\013"
    _globals["_GRAPHSERVICE"].methods_by_name["GetCodegenFeaturesFromGraph"]._options = None
    _globals["_GRAPHSERVICE"].methods_by_name[
        "GetCodegenFeaturesFromGraph"
    ]._serialized_options = b"\220\002\001\200}\013"
    _globals["_FEATURESQL"]._serialized_start = 206
    _globals["_FEATURESQL"]._serialized_end = 806
    _globals["_GETFEATURESQLRESPONSE"]._serialized_start = 808
    _globals["_GETFEATURESQLRESPONSE"]._serialized_end = 888
    _globals["_GETFEATURESQLREQUEST"]._serialized_start = 890
    _globals["_GETFEATURESQLREQUEST"]._serialized_end = 949
    _globals["_FEATUREMETADATA"]._serialized_start = 952
    _globals["_FEATUREMETADATA"]._serialized_end = 1342
    _globals["_GETFEATURESMETADATARESPONSE"]._serialized_start = 1345
    _globals["_GETFEATURESMETADATARESPONSE"]._serialized_end = 1512
    _globals["_GETFEATURESMETADATAREQUEST"]._serialized_start = 1514
    _globals["_GETFEATURESMETADATAREQUEST"]._serialized_end = 1575
    _globals["_UPDATEGRAPHREQUEST"]._serialized_start = 1578
    _globals["_UPDATEGRAPHREQUEST"]._serialized_end = 1808
    _globals["_UPDATEGRAPHRESPONSE"]._serialized_start = 1810
    _globals["_UPDATEGRAPHRESPONSE"]._serialized_end = 1831
    _globals["_GETGRAPHREQUEST"]._serialized_start = 1833
    _globals["_GETGRAPHREQUEST"]._serialized_end = 1887
    _globals["_GETGRAPHRESPONSE"]._serialized_start = 1890
    _globals["_GETGRAPHRESPONSE"]._serialized_end = 2118
    _globals["_PYTHONVERSION"]._serialized_start = 2120
    _globals["_PYTHONVERSION"]._serialized_end = 2201
    _globals["_GETCODEGENFEATURESFROMGRAPHREQUEST"]._serialized_start = 2204
    _globals["_GETCODEGENFEATURESFROMGRAPHREQUEST"]._serialized_end = 2435
    _globals["_GETCODEGENFEATURESFROMGRAPHRESPONSE"]._serialized_start = 2437
    _globals["_GETCODEGENFEATURESFROMGRAPHRESPONSE"]._serialized_end = 2553
    _globals["_GRAPHSERVICE"]._serialized_start = 2556
    _globals["_GRAPHSERVICE"]._serialized_end = 3127
# @@protoc_insertion_point(module_scope)



from rich.style import Style
from rich.theme import Theme

from chalk._reporting.rich.color import PASTELY_CYAN, SERENDIPITOUS_PURPLE

CHALK_THEME = Theme(
    {
        "progress.elapsed": Style(color=SERENDIPITOUS_PURPLE),
        "progress.download": Style(color=PASTELY_CYAN),
        "bar.pulse": Style(color=SERENDIPITOUS_PURPLE),
        "progress.percentage": Style(color=PASTELY_CYAN),
    }
)



from chalk._gen.chalk.auth.v1 import permissions_pb2 as _permissions_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class DatasetRevisionStatus(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    DATASET_REVISION_STATUS_UNSPECIFIED: _ClassVar[DatasetRevisionStatus]
    DATASET_REVISION_STATUS_UNKNOWN: _ClassVar[DatasetRevisionStatus]
    DATASET_REVISION_STATUS_WORKING: _ClassVar[DatasetRevisionStatus]
    DATASET_REVISION_STATUS_COMPLETED: _ClassVar[DatasetRevisionStatus]
    DATASET_REVISION_STATUS_FAILED: _ClassVar[DatasetRevisionStatus]
    DATASET_REVISION_STATUS_CANCELED: _ClassVar[DatasetRevisionStatus]
    DATASET_REVISION_STATUS_QUEUED: _ClassVar[DatasetRevisionStatus]

class DatasetVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    DATASET_VERSION_UNSPECIFIED: _ClassVar[DatasetVersion]
    DATASET_VERSION_UNKNOWN: _ClassVar[DatasetVersion]
    DATASET_VERSION_BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES: _ClassVar[DatasetVersion]
    DATASET_VERSION_DATASET_WRITER: _ClassVar[DatasetVersion]
    DATASET_VERSION_BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2: _ClassVar[DatasetVersion]
    DATASET_VERSION_COMPUTE_RESOLVER_OUTPUT_V1: _ClassVar[DatasetVersion]
    DATASET_VERSION_NATIVE_DTYPES: _ClassVar[DatasetVersion]
    DATASET_VERSION_NATIVE_COLUMN_NAMES: _ClassVar[DatasetVersion]

class OfflineQueryGivensVersion(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    OFFLINE_QUERY_GIVENS_VERSION_UNSPECIFIED: _ClassVar[OfflineQueryGivensVersion]
    OFFLINE_QUERY_GIVENS_VERSION_UNKNOWN: _ClassVar[OfflineQueryGivensVersion]
    OFFLINE_QUERY_GIVENS_VERSION_NATIVE_TS_FEATURE_FOR_ROOT_NS: _ClassVar[OfflineQueryGivensVersion]
    OFFLINE_QUERY_GIVENS_VERSION_SINGLE_TS_COL_NAME: _ClassVar[OfflineQueryGivensVersion]
    OFFLINE_QUERY_GIVENS_VERSION_SINGLE_TS_COL_NAME_WITH_URI_PREFIX: _ClassVar[OfflineQueryGivensVersion]

DATASET_REVISION_STATUS_UNSPECIFIED: DatasetRevisionStatus
DATASET_REVISION_STATUS_UNKNOWN: DatasetRevisionStatus
DATASET_REVISION_STATUS_WORKING: DatasetRevisionStatus
DATASET_REVISION_STATUS_COMPLETED: DatasetRevisionStatus
DATASET_REVISION_STATUS_FAILED: DatasetRevisionStatus
DATASET_REVISION_STATUS_CANCELED: DatasetRevisionStatus
DATASET_REVISION_STATUS_QUEUED: DatasetRevisionStatus
DATASET_VERSION_UNSPECIFIED: DatasetVersion
DATASET_VERSION_UNKNOWN: DatasetVersion
DATASET_VERSION_BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES: DatasetVersion
DATASET_VERSION_DATASET_WRITER: DatasetVersion
DATASET_VERSION_BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2: DatasetVersion
DATASET_VERSION_COMPUTE_RESOLVER_OUTPUT_V1: DatasetVersion
DATASET_VERSION_NATIVE_DTYPES: DatasetVersion
DATASET_VERSION_NATIVE_COLUMN_NAMES: DatasetVersion
OFFLINE_QUERY_GIVENS_VERSION_UNSPECIFIED: OfflineQueryGivensVersion
OFFLINE_QUERY_GIVENS_VERSION_UNKNOWN: OfflineQueryGivensVersion
OFFLINE_QUERY_GIVENS_VERSION_NATIVE_TS_FEATURE_FOR_ROOT_NS: OfflineQueryGivensVersion
OFFLINE_QUERY_GIVENS_VERSION_SINGLE_TS_COL_NAME: OfflineQueryGivensVersion
OFFLINE_QUERY_GIVENS_VERSION_SINGLE_TS_COL_NAME_WITH_URI_PREFIX: OfflineQueryGivensVersion

class DatasetRevisionMeta(_message.Message):
    __slots__ = (
        "numeric_id",
        "offline_query_id",
        "dataset_id",
        "givens_uri",
        "givens_version",
        "output_uri",
        "output_version",
        "branch_name",
        "num_rows",
        "physical_size_bytes",
        "output_columns",
        "output_fqns",
        "agent_id",
        "completed_at",
        "num_shards",
        "num_computers",
        "metadata",
        "status",
        "num_rows_calculated",
        "physical_size_bytes_calculated",
        "created_at",
    )
    NUMERIC_ID_FIELD_NUMBER: _ClassVar[int]
    OFFLINE_QUERY_ID_FIELD_NUMBER: _ClassVar[int]
    DATASET_ID_FIELD_NUMBER: _ClassVar[int]
    GIVENS_URI_FIELD_NUMBER: _ClassVar[int]
    GIVENS_VERSION_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_URI_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_VERSION_FIELD_NUMBER: _ClassVar[int]
    BRANCH_NAME_FIELD_NUMBER: _ClassVar[int]
    NUM_ROWS_FIELD_NUMBER: _ClassVar[int]
    PHYSICAL_SIZE_BYTES_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    OUTPUT_FQNS_FIELD_NUMBER: _ClassVar[int]
    AGENT_ID_FIELD_NUMBER: _ClassVar[int]
    COMPLETED_AT_FIELD_NUMBER: _ClassVar[int]
    NUM_SHARDS_FIELD_NUMBER: _ClassVar[int]
    NUM_COMPUTERS_FIELD_NUMBER: _ClassVar[int]
    METADATA_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    NUM_ROWS_CALCULATED_FIELD_NUMBER: _ClassVar[int]
    PHYSICAL_SIZE_BYTES_CALCULATED_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    numeric_id: int
    offline_query_id: str
    dataset_id: str
    givens_uri: str
    givens_version: OfflineQueryGivensVersion
    output_uri: str
    output_version: DatasetVersion
    branch_name: str
    num_rows: int
    physical_size_bytes: int
    output_columns: _containers.RepeatedScalarFieldContainer[str]
    output_fqns: _containers.RepeatedScalarFieldContainer[str]
    agent_id: str
    completed_at: _timestamp_pb2.Timestamp
    num_shards: int
    num_computers: int
    metadata: _struct_pb2.Value
    status: DatasetRevisionStatus
    num_rows_calculated: int
    physical_size_bytes_calculated: int
    created_at: _timestamp_pb2.Timestamp
    def __init__(
        self,
        numeric_id: _Optional[int] = ...,
        offline_query_id: _Optional[str] = ...,
        dataset_id: _Optional[str] = ...,
        givens_uri: _Optional[str] = ...,
        givens_version: _Optional[_Union[OfflineQueryGivensVersion, str]] = ...,
        output_uri: _Optional[str] = ...,
        output_version: _Optional[_Union[DatasetVersion, str]] = ...,
        branch_name: _Optional[str] = ...,
        num_rows: _Optional[int] = ...,
        physical_size_bytes: _Optional[int] = ...,
        output_columns: _Optional[_Iterable[str]] = ...,
        output_fqns: _Optional[_Iterable[str]] = ...,
        agent_id: _Optional[str] = ...,
        completed_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        num_shards: _Optional[int] = ...,
        num_computers: _Optional[int] = ...,
        metadata: _Optional[_Union[_struct_pb2.Value, _Mapping]] = ...,
        status: _Optional[_Union[DatasetRevisionStatus, str]] = ...,
        num_rows_calculated: _Optional[int] = ...,
        physical_size_bytes_calculated: _Optional[int] = ...,
        created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
    ) -> None: ...

class DatasetMeta(_message.Message):
    __slots__ = ("id", "environment_id", "dataset_name", "created_at", "most_recent_revision")
    ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    DATASET_NAME_FIELD_NUMBER: _ClassVar[int]
    CREATED_AT_FIELD_NUMBER: _ClassVar[int]
    MOST_RECENT_REVISION_FIELD_NUMBER: _ClassVar[int]
    id: str
    environment_id: str
    dataset_name: str
    created_at: _timestamp_pb2.Timestamp
    most_recent_revision: DatasetRevisionMeta
    def __init__(
        self,
        id: _Optional[str] = ...,
        environment_id: _Optional[str] = ...,
        dataset_name: _Optional[str] = ...,
        created_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...,
        most_recent_revision: _Optional[_Union[DatasetRevisionMeta, _Mapping]] = ...,
    ) -> None: ...

class ListDatasetsRequest(_message.Message):
    __slots__ = ("cursor", "limit", "search")
    CURSOR_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    SEARCH_FIELD_NUMBER: _ClassVar[int]
    cursor: str
    limit: int
    search: str
    def __init__(
        self, cursor: _Optional[str] = ..., limit: _Optional[int] = ..., search: _Optional[str] = ...
    ) -> None: ...

class ListDatasetsResponse(_message.Message):
    __slots__ = ("datasets", "cursor")
    DATASETS_FIELD_NUMBER: _ClassVar[int]
    CURSOR_FIELD_NUMBER: _ClassVar[int]
    datasets: _containers.RepeatedCompositeFieldContainer[DatasetMeta]
    cursor: str
    def __init__(
        self, datasets: _Optional[_Iterable[_Union[DatasetMeta, _Mapping]]] = ..., cursor: _Optional[str] = ...
    ) -> None: ...

class GetDatasetRequest(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class GetDatasetResponse(_message.Message):
    __slots__ = ("dataset",)
    DATASET_FIELD_NUMBER: _ClassVar[int]
    dataset: DatasetMeta
    def __init__(self, dataset: _Optional[_Union[DatasetMeta, _Mapping]] = ...) -> None: ...

class ListDatasetRevisionsRequest(_message.Message):
    __slots__ = ("dataset_id", "cursor", "limit")
    DATASET_ID_FIELD_NUMBER: _ClassVar[int]
    CURSOR_FIELD_NUMBER: _ClassVar[int]
    LIMIT_FIELD_NUMBER: _ClassVar[int]
    dataset_id: str
    cursor: str
    limit: int
    def __init__(
        self, dataset_id: _Optional[str] = ..., cursor: _Optional[str] = ..., limit: _Optional[int] = ...
    ) -> None: ...

class ListDatasetRevisionsResponse(_message.Message):
    __slots__ = ("revisions", "cursor")
    REVISIONS_FIELD_NUMBER: _ClassVar[int]
    CURSOR_FIELD_NUMBER: _ClassVar[int]
    revisions: _containers.RepeatedCompositeFieldContainer[DatasetRevisionMeta]
    cursor: str
    def __init__(
        self, revisions: _Optional[_Iterable[_Union[DatasetRevisionMeta, _Mapping]]] = ..., cursor: _Optional[str] = ...
    ) -> None: ...

class GetDatasetRevisionRequest(_message.Message):
    __slots__ = ("id",)
    ID_FIELD_NUMBER: _ClassVar[int]
    id: str
    def __init__(self, id: _Optional[str] = ...) -> None: ...

class GetDatasetRevisionResponse(_message.Message):
    __slots__ = ("revision",)
    REVISION_FIELD_NUMBER: _ClassVar[int]
    revision: DatasetRevisionMeta
    def __init__(self, revision: _Optional[_Union[DatasetRevisionMeta, _Mapping]] = ...) -> None: ...



import dataclasses
import datetime as dt
import enum
import sys
import traceback
from typing import Any, List, Optional, Set, Tuple, Type, Union

import dataclasses_json

from chalk import Resolver
from chalk.features import Feature, Features, is_features_cls
from chalk.parsed.branch_state_rich import FeatureFieldRichSummary, ResolverRichSummary
from chalk.parsed.duplicate_input_gql import FailedImport, UpsertFeatureGQL, UpsertGraphGQL, UpsertResolverGQL
from chalk.parsed.json_conversions import convert_type_to_gql
from chalk.utils.collections import get_unique_item
from chalk.utils.log_with_context import get_logger

_logger = get_logger(__name__)


class EntityUpdateSource(enum.Enum):
    live_update = "live_update"
    deployment_source = "deployment_source"


class EntityUpdateType(enum.Enum):
    resolver = "resolver"
    feature_class = "feature_class"


# Need this to convert datetimes via ISO instead of numberic timestamp -- deser wasn't working otherwise.
DATACLASSES_JSON_TIMESTAMP_FIELD = dataclasses.field(
    metadata=dataclasses_json.config(
        encoder=dt.datetime.isoformat,
        decoder=dt.datetime.fromisoformat,
    )
)


@dataclasses_json.dataclass_json
@dataclasses.dataclass
class BlobLoadError:
    traceback: str
    message: str


@dataclasses_json.dataclass_json
@dataclasses.dataclass
class EntityUpdate:
    """
    Represents a record of loading a pickled resolver/feature update on the branch server.
    Either produces a resolver/feature that gets added to the graph, or an error message.
    """

    blob_id: str
    source: EntityUpdateSource
    success: bool  # True if this entity was unpickled & added to the graph w/o errors
    # TODO (rkargon) This currently shows the time the blob was loaded by the current active worker - not the time it was created/sent.
    timestamp: dt.datetime = DATACLASSES_JSON_TIMESTAMP_FIELD

    # These might be None depending on whether an error was raised when updating the graph
    name: Optional[str] = None  # either resolver.fqn or feature_class.name
    entity_type: Optional[EntityUpdateType] = None
    resolver_gql: Optional[UpsertResolverGQL] = None
    feature_fields_gql: Optional[List[UpsertFeatureGQL]] = None
    error: Optional[BlobLoadError] = None

    @classmethod
    def from_obj(cls, blob_id: str, obj: Union[Resolver, Type[Features]]):
        kwargs: dict[str, Any] = {
            "blob_id": blob_id,
            "timestamp": dt.datetime.now(tz=dt.timezone.utc),
            "source": EntityUpdateSource.live_update,
            "success": True,
        }
        gql = None

        if isinstance(obj, Resolver):
            try:
                gql = convert_type_to_gql(obj)
            except Exception:
                _logger.error(f"Failed to convert {obj=} with {blob_id=} to GQL.", exc_info=True)
            kwargs.update(
                {
                    "name": obj.fqn,
                    "entity_type": EntityUpdateType.resolver,
                    "resolver_gql": gql,
                    "feature_fields_gql": None,
                }
            )
        elif is_features_cls(obj):
            try:
                gql = [convert_type_to_gql(f) for f in obj.features if not f.is_autogenerated]
            except Exception:
                _logger.error(f"Failed to convert {obj=} with {blob_id=} to GQL.", exc_info=True)
            kwargs.update(
                {
                    "name": obj.namespace,
                    "entity_type": EntityUpdateType.feature_class,
                    "resolver_gql": None,
                    "feature_fields_gql": gql,
                }
            )
        else:
            raise ValueError(f"Unrecognized object type: {obj}")
        return cls(**kwargs)

    @classmethod
    def from_exc(cls, blob_id: str, exc: Exception):
        # need to build these explicitly in 3.8
        exc_type, exc_value, exc_traceback = sys.exc_info()
        trace_lines = traceback.format_exception(exc_type, value=exc_value, tb=exc_traceback)

        err = BlobLoadError(
            traceback="".join(trace_lines),
            message=str(exc),
        )
        return cls(
            blob_id=blob_id,
            timestamp=dt.datetime.now(tz=dt.timezone.utc),
            source=EntityUpdateSource.live_update,
            error=err,
            success=False,
        )


@dataclasses_json.dataclass_json
@dataclasses.dataclass
class BranchGraphSummary:
    """
    Summary of the state of the feature/resolver graph on a branch deployment.
    This includes both data from the deployment source and from live updates sent
    from e.g. Jupyter notebooks.
    """

    # includes all features/resolvers currently used to run queries, as well as any validation errors/warnings
    current_graph_gql: Optional[UpsertGraphGQL]

    # Errors caused when importing the deployment source
    import_errors: List[FailedImport]

    # Result or error of each pickled entity update, in chronological order.
    blob_history: List[EntityUpdate]

    # Name's of feature classes currently in the graph that came from pickled blobs
    # i.e. these names are features that _differ_ from the deployment source
    blob_feature_classes: Set[str]

    # FQN's of resolvers currently in the graph that came from pickled blobs
    # i.e. these names are resolvers that _differ_ from the deployment source
    blob_resolvers: Set[str]

    # Need the time the deployment was created in order to display updated_at for the graph
    deployment_created: Optional[str] = None

    def resolver(self, ref: Tuple[Resolver, str]) -> Optional[ResolverRichSummary]:
        if isinstance(ref, Resolver):
            fqn = ref.fqn
        elif isinstance(ref, str):
            fqn = ref
        else:
            raise ValueError(f"Unrecognized type: {ref}")
        short_name = _short_name(fqn)

        if not self.current_graph_gql:
            return None
        assert self.current_graph_gql.resolvers is not None
        resolver = get_unique_item(x for x in self.current_graph_gql.resolvers if _short_name(x.fqn) == short_name)
        update_time = self._get_last_updated_time(EntityUpdateType.resolver, name=resolver.fqn)
        return ResolverRichSummary(resolver, update_time)

    def feature(self, ref: Tuple[Feature, str]) -> Optional[FeatureFieldRichSummary]:
        if isinstance(ref, Feature):
            fqn = ref.fqn
        elif isinstance(ref, str):
            fqn = ref
        else:
            raise ValueError(f"Unrecognized type: {ref}")
        if not self.current_graph_gql:
            return None
        assert self.current_graph_gql.features is not None
        feature_field = get_unique_item(x for x in self.current_graph_gql.features if x.id.fqn == fqn)
        update_time = self._get_last_updated_time(EntityUpdateType.resolver, name=feature_field.id.namespace)
        return FeatureFieldRichSummary(feature_field, update_time)

    # def features_class(self, ref: Tuple[Features, str]) -> Optional["FeaturesClassRichSummary"]:
    #     pass

    # def entity(self, ref: Union[str, Feature, Resolver, Features]):
    #     return self.get(ref)
    #
    # def get(self, ref: Union[str, Feature, Resolver, Features]):
    #     pass

    def _get_last_updated_time(self, entity_type: EntityUpdateType, name: str) -> Optional[dt.datetime]:
        for update in reversed(self.blob_history):
            if not update.success:
                continue
            if update.name == name and update.entity_type == entity_type:
                return _blob_id_to_timestamp(update.blob_id)
        return None


def _short_name(fqn: str):
    return fqn.split(".")[-1]


def _blob_id_to_timestamp(blob_id: str) -> dt.datetime:
    """
    TODO (rkargon) - this relies on the implementation of cuid(), instead we should store the 'blob created timestamp' in the EntityUpdate
    """
    encoding = "0123456789abcdefghijklmnopqrstuvwxyz"

    def _str_to_timestamp(s: str):
        """Assumes `s` is a base-len(encoding) encoded integer."""
        v = 0
        for x in s:
            v *= len(encoding)
            v += encoding.index(x)
        return v

    time_str = blob_id[1:9]
    timestamp = _str_to_timestamp(time_str) // 1000
    return dt.datetime.fromtimestamp(timestamp, tz=dt.timezone.utc)



from __future__ import annotations

from datetime import date, datetime, time, timedelta
from typing import Dict, Mapping, NewType, Sequence, Tuple, Type, Union

import pyarrow as pa

__all__ = ["TPrimitive", "ChalkStructType"]


class ChalkStructType(type):
    __chalk_type_hints__: Dict[str, Type[TPrimitive]]  # pyright: ignore[reportUninitializedInstanceVariable]

    def __new__(cls, name: str, bases: Tuple[Type], annotations: Dict[str, Type[TPrimitive]]):
        instance = super().__new__(cls, name, bases, annotations)
        instance.__chalk_type_hints__ = annotations
        return instance


TPrimitiveArrowScalar = NewType("TPrimitiveArrowScalar", pa.Scalar)  # pyright: ignore[reportGeneralTypeIssues]
"""
`pa.Scalar` is used for internal logic inside of many primitive-transforming functions.
In order to prevent accidental misuse of an internal scalar value with an actual output,
`TPrimitiveArrowScalar` is declared as a newtype.
"""

TPrimitive = Union[
    None,
    str,
    int,
    float,
    bool,
    date,
    datetime,
    time,
    timedelta,
    Sequence["TPrimitive"],
    Mapping[str, "TPrimitive"],
    Mapping["TPrimitive", "TPrimitive"],
    ChalkStructType,
    TPrimitiveArrowScalar,
]



from typing import Collection


def tag_matches(datasource_tag: str, context_tags: Collection[str]) -> bool:
    for t in context_tags:
        if t == datasource_tag:
            return True
        split = t.split(":")
        if len(split) == 2 and split[0] == datasource_tag:
            return True
    return False



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class KubernetesNodeTaint(_message.Message):
    __slots__ = ("key", "value", "effect", "time_added")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    EFFECT_FIELD_NUMBER: _ClassVar[int]
    TIME_ADDED_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    effect: str
    time_added: int
    def __init__(
        self,
        key: _Optional[str] = ...,
        value: _Optional[str] = ...,
        effect: _Optional[str] = ...,
        time_added: _Optional[int] = ...,
    ) -> None: ...

class KubernetesNodeSpec(_message.Message):
    __slots__ = ("pod_cidr", "taints", "pod_cidrs", "provider_id", "unschedulable")
    POD_CIDR_FIELD_NUMBER: _ClassVar[int]
    TAINTS_FIELD_NUMBER: _ClassVar[int]
    POD_CIDRS_FIELD_NUMBER: _ClassVar[int]
    PROVIDER_ID_FIELD_NUMBER: _ClassVar[int]
    UNSCHEDULABLE_FIELD_NUMBER: _ClassVar[int]
    pod_cidr: str
    taints: _containers.RepeatedCompositeFieldContainer[KubernetesNodeTaint]
    pod_cidrs: _containers.RepeatedScalarFieldContainer[str]
    provider_id: str
    unschedulable: bool
    def __init__(
        self,
        pod_cidr: _Optional[str] = ...,
        taints: _Optional[_Iterable[_Union[KubernetesNodeTaint, _Mapping]]] = ...,
        pod_cidrs: _Optional[_Iterable[str]] = ...,
        provider_id: _Optional[str] = ...,
        unschedulable: bool = ...,
    ) -> None: ...

class KubernetesNodeCondition(_message.Message):
    __slots__ = ("type", "status", "last_heartbeat_time", "last_transition_time", "reason", "message")
    TYPE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    LAST_HEARTBEAT_TIME_FIELD_NUMBER: _ClassVar[int]
    LAST_TRANSITION_TIME_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    type: str
    status: str
    last_heartbeat_time: int
    last_transition_time: int
    reason: str
    message: str
    def __init__(
        self,
        type: _Optional[str] = ...,
        status: _Optional[str] = ...,
        last_heartbeat_time: _Optional[int] = ...,
        last_transition_time: _Optional[int] = ...,
        reason: _Optional[str] = ...,
        message: _Optional[str] = ...,
    ) -> None: ...

class KubernetesAttachedVolume(_message.Message):
    __slots__ = ("name", "device_path")
    NAME_FIELD_NUMBER: _ClassVar[int]
    DEVICE_PATH_FIELD_NUMBER: _ClassVar[int]
    name: str
    device_path: str
    def __init__(self, name: _Optional[str] = ..., device_path: _Optional[str] = ...) -> None: ...

class KubernetesNodeStatus(_message.Message):
    __slots__ = ("conditions", "volumes_in_use", "volumes_attached")
    CONDITIONS_FIELD_NUMBER: _ClassVar[int]
    VOLUMES_IN_USE_FIELD_NUMBER: _ClassVar[int]
    VOLUMES_ATTACHED_FIELD_NUMBER: _ClassVar[int]
    conditions: _containers.RepeatedCompositeFieldContainer[KubernetesNodeCondition]
    volumes_in_use: _containers.RepeatedScalarFieldContainer[str]
    volumes_attached: _containers.RepeatedCompositeFieldContainer[KubernetesAttachedVolume]
    def __init__(
        self,
        conditions: _Optional[_Iterable[_Union[KubernetesNodeCondition, _Mapping]]] = ...,
        volumes_in_use: _Optional[_Iterable[str]] = ...,
        volumes_attached: _Optional[_Iterable[_Union[KubernetesAttachedVolume, _Mapping]]] = ...,
    ) -> None: ...

class KubernetesNodeData(_message.Message):
    __slots__ = (
        "team",
        "name",
        "uid",
        "instance_type",
        "region",
        "zone",
        "creation_timestamp",
        "deletion_timestamp",
        "observed_timestamp",
        "labels",
        "annotations",
        "machine_id",
        "system_uuid",
        "boot_id",
        "unschedulable",
        "namespace",
        "instance_id",
        "cluster",
        "total_cpu",
        "total_memory",
        "allocatable_cpu",
        "allocatable_memory",
        "spec",
        "status",
    )
    class LabelsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    class AnnotationsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...

    TEAM_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    UID_FIELD_NUMBER: _ClassVar[int]
    INSTANCE_TYPE_FIELD_NUMBER: _ClassVar[int]
    REGION_FIELD_NUMBER: _ClassVar[int]
    ZONE_FIELD_NUMBER: _ClassVar[int]
    CREATION_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    DELETION_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    OBSERVED_TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    LABELS_FIELD_NUMBER: _ClassVar[int]
    ANNOTATIONS_FIELD_NUMBER: _ClassVar[int]
    MACHINE_ID_FIELD_NUMBER: _ClassVar[int]
    SYSTEM_UUID_FIELD_NUMBER: _ClassVar[int]
    BOOT_ID_FIELD_NUMBER: _ClassVar[int]
    UNSCHEDULABLE_FIELD_NUMBER: _ClassVar[int]
    NAMESPACE_FIELD_NUMBER: _ClassVar[int]
    INSTANCE_ID_FIELD_NUMBER: _ClassVar[int]
    CLUSTER_FIELD_NUMBER: _ClassVar[int]
    TOTAL_CPU_FIELD_NUMBER: _ClassVar[int]
    TOTAL_MEMORY_FIELD_NUMBER: _ClassVar[int]
    ALLOCATABLE_CPU_FIELD_NUMBER: _ClassVar[int]
    ALLOCATABLE_MEMORY_FIELD_NUMBER: _ClassVar[int]
    SPEC_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    team: str
    name: str
    uid: str
    instance_type: str
    region: str
    zone: str
    creation_timestamp: int
    deletion_timestamp: int
    observed_timestamp: int
    labels: _containers.ScalarMap[str, str]
    annotations: _containers.ScalarMap[str, str]
    machine_id: str
    system_uuid: str
    boot_id: str
    unschedulable: bool
    namespace: str
    instance_id: str
    cluster: str
    total_cpu: str
    total_memory: str
    allocatable_cpu: str
    allocatable_memory: str
    spec: KubernetesNodeSpec
    status: KubernetesNodeStatus
    def __init__(
        self,
        team: _Optional[str] = ...,
        name: _Optional[str] = ...,
        uid: _Optional[str] = ...,
        instance_type: _Optional[str] = ...,
        region: _Optional[str] = ...,
        zone: _Optional[str] = ...,
        creation_timestamp: _Optional[int] = ...,
        deletion_timestamp: _Optional[int] = ...,
        observed_timestamp: _Optional[int] = ...,
        labels: _Optional[_Mapping[str, str]] = ...,
        annotations: _Optional[_Mapping[str, str]] = ...,
        machine_id: _Optional[str] = ...,
        system_uuid: _Optional[str] = ...,
        boot_id: _Optional[str] = ...,
        unschedulable: bool = ...,
        namespace: _Optional[str] = ...,
        instance_id: _Optional[str] = ...,
        cluster: _Optional[str] = ...,
        total_cpu: _Optional[str] = ...,
        total_memory: _Optional[str] = ...,
        allocatable_cpu: _Optional[str] = ...,
        allocatable_memory: _Optional[str] = ...,
        spec: _Optional[_Union[KubernetesNodeSpec, _Mapping]] = ...,
        status: _Optional[_Union[KubernetesNodeStatus, _Mapping]] = ...,
    ) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/audit.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import agent_pb2 as chalk_dot_auth_dot_v1_dot_agent__pb2
from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.rpc import code_pb2 as google_dot_rpc_dot_code__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x1b\x63halk/server/v1/audit.proto\x12\x0f\x63halk.server.v1\x1a\x19\x63halk/auth/v1/agent.proto\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x15google/rpc/code.proto"\x87\x05\n\x08\x41uditLog\x12*\n\x05\x61gent\x18\x01 \x01(\x0b\x32\x14.chalk.auth.v1.AgentR\x05\x61gent\x12%\n\x0b\x64\x65scription\x18\x02 \x01(\tH\x00R\x0b\x64\x65scription\x88\x01\x01\x12\x1a\n\x08\x65ndpoint\x18\x03 \x01(\tR\x08\x65ndpoint\x12*\n\x02\x61t\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x02\x61t\x12\x1e\n\x08trace_id\x18\x05 \x01(\x04H\x01R\x07traceId\x88\x01\x01\x12)\n\x04\x63ode\x18\x06 \x01(\x0e\x32\x10.google.rpc.CodeH\x02R\x04\x63ode\x88\x01\x01\x12@\n\x07request\x18\x07 \x03(\x0b\x32&.chalk.server.v1.AuditLog.RequestEntryR\x07request\x12\x43\n\x08response\x18\x08 \x03(\x0b\x32\'.chalk.server.v1.AuditLog.ResponseEntryR\x08response\x12\x13\n\x02ip\x18\t \x01(\tH\x03R\x02ip\x88\x01\x01\x12\x19\n\x05\x65rror\x18\n \x01(\tH\x04R\x05\x65rror\x88\x01\x01\x1aR\n\x0cRequestEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x1aS\n\rResponseEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12,\n\x05value\x18\x02 \x01(\x0b\x32\x16.google.protobuf.ValueR\x05value:\x02\x38\x01\x42\x0e\n\x0c_descriptionB\x0b\n\t_trace_idB\x07\n\x05_codeB\x05\n\x03_ipB\x08\n\x06_error"\xd6\x01\n\x13GetAuditLogsRequest\x12>\n\nstart_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\tstartTime\x88\x01\x01\x12:\n\x08\x65nd_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x01R\x07\x65ndTime\x88\x01\x01\x12\'\n\x0f\x65ndpoint_filter\x18\x03 \x03(\tR\x0e\x65ndpointFilterB\r\n\x0b_start_timeB\x0b\n\t_end_time"E\n\x14GetAuditLogsResponse\x12-\n\x04logs\x18\x01 \x03(\x0b\x32\x19.chalk.server.v1.AuditLogR\x04logs2p\n\x0c\x41uditService\x12`\n\x0cGetAuditLogs\x12$.chalk.server.v1.GetAuditLogsRequest\x1a%.chalk.server.v1.GetAuditLogsResponse"\x03\x80}\x06\x42\x93\x01\n\x13\x63om.chalk.server.v1B\nAuditProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.audit_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\nAuditProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_AUDITLOG_REQUESTENTRY"]._options = None
    _globals["_AUDITLOG_REQUESTENTRY"]._serialized_options = b"8\001"
    _globals["_AUDITLOG_RESPONSEENTRY"]._options = None
    _globals["_AUDITLOG_RESPONSEENTRY"]._serialized_options = b"8\001"
    _globals["_AUDITSERVICE"].methods_by_name["GetAuditLogs"]._options = None
    _globals["_AUDITSERVICE"].methods_by_name["GetAuditLogs"]._serialized_options = b"\200}\006"
    _globals["_AUDITLOG"]._serialized_start = 195
    _globals["_AUDITLOG"]._serialized_end = 842
    _globals["_AUDITLOG_REQUESTENTRY"]._serialized_start = 620
    _globals["_AUDITLOG_REQUESTENTRY"]._serialized_end = 702
    _globals["_AUDITLOG_RESPONSEENTRY"]._serialized_start = 704
    _globals["_AUDITLOG_RESPONSEENTRY"]._serialized_end = 787
    _globals["_GETAUDITLOGSREQUEST"]._serialized_start = 845
    _globals["_GETAUDITLOGSREQUEST"]._serialized_end = 1059
    _globals["_GETAUDITLOGSRESPONSE"]._serialized_start = 1061
    _globals["_GETAUDITLOGSRESPONSE"]._serialized_end = 1130
    _globals["_AUDITSERVICE"]._serialized_start = 1132
    _globals["_AUDITSERVICE"]._serialized_end = 1244
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/server/v1/scheduled_query.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from chalk._gen.chalk.auth.v1 import permissions_pb2 as chalk_dot_auth_dot_v1_dot_permissions__pb2
from chalk._gen.chalk.server.v1 import scheduled_query_run_pb2 as chalk_dot_server_dot_v1_dot_scheduled__query__run__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n%chalk/server/v1/scheduled_query.proto\x12\x0f\x63halk.server.v1\x1a\x1f\x63halk/auth/v1/permissions.proto\x1a)chalk/server/v1/scheduled_query_run.proto2\x91\x01\n\x15ScheduledQueryService\x12x\n\x14GetScheduledQueryRun\x12,.chalk.server.v1.GetScheduledQueryRunRequest\x1a-.chalk.server.v1.GetScheduledQueryRunResponse"\x03\x80}\x06\x42\x9c\x01\n\x13\x63om.chalk.server.v1B\x13ScheduledQueryProtoP\x01Z\x12server/v1;serverv1\xa2\x02\x03\x43SX\xaa\x02\x0f\x43halk.Server.V1\xca\x02\x0f\x43halk\\Server\\V1\xe2\x02\x1b\x43halk\\Server\\V1\\GPBMetadata\xea\x02\x11\x43halk::Server::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.server.v1.scheduled_query_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\023com.chalk.server.v1B\023ScheduledQueryProtoP\001Z\022server/v1;serverv1\242\002\003CSX\252\002\017Chalk.Server.V1\312\002\017Chalk\\Server\\V1\342\002\033Chalk\\Server\\V1\\GPBMetadata\352\002\021Chalk::Server::V1"
    _globals["_SCHEDULEDQUERYSERVICE"].methods_by_name["GetScheduledQueryRun"]._options = None
    _globals["_SCHEDULEDQUERYSERVICE"].methods_by_name["GetScheduledQueryRun"]._serialized_options = b"\200}\006"
    _globals["_SCHEDULEDQUERYSERVICE"]._serialized_start = 135
    _globals["_SCHEDULEDQUERYSERVICE"]._serialized_end = 280
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from typing import Mapping

from typing_extensions import final

from chalk.operators import StaticOperator


@final
class RenameOperator(StaticOperator):
    _chalk__operator_name = StaticOperator._chalk__operator_prefix + "rename_columns"

    def __init__(
        self,
        parent: StaticOperator,
        column_names: tuple[str, ...],
        old_name_to_new_name_map: Mapping[str, str],
    ):
        super().__init__(
            parent=parent,
            column_names=column_names,
            old_name_to_new_name_map=old_name_to_new_name_map,
        )
        self.column_names = column_names
        self.parent = parent
        self.old_name_to_new_name_map = old_name_to_new_name_map



GRPC_RESULT_METADATA_COL_PREFIX: str = "__chalk__.__result_metadata__."
"""
Column name prefix in which the GRPC server returns metadata about a given feature
(e.g. is it a cache hit, what the resolver fqn was, etc.)
"""



from __future__ import annotations

import os
from os import PathLike
from typing import TYPE_CHECKING, Any, Dict, Optional, TypeVar, Union

import duckdb
from sqlalchemy.engine import Engine

from chalk.sql._internal.sql_source import BaseSQLSource, SQLSourceKind, TableIngestMixIn
from chalk.sql.protocols import SQLSourceWithTableIngestProtocol

if TYPE_CHECKING:
    import pyarrow as pa
    from sqlalchemy.engine.url import URL

T = TypeVar("T")


class DuckDBSourceImpl(TableIngestMixIn, BaseSQLSource, SQLSourceWithTableIngestProtocol):
    kind = SQLSourceKind.duckdb

    def __init__(
        self,
        name: Optional[str] = None,
        filename: Optional[Union[PathLike, str]] = None,
        engine_args: Optional[Dict[str, Any]] = None,
        async_engine_args: Optional[Dict[str, Any]] = None,
        arrow_tables: dict[str, pa.Table] | None = None,
    ):
        self.ingested_tables: Dict[str, Any] = {}
        self.filename = filename
        if engine_args is None:
            engine_args = {}
        if async_engine_args is None:
            async_engine_args = {}
        if arrow_tables is None:
            arrow_tables = {}

        # We set the default isolation level to autocommit since the SQL sources are read-only, and thus
        # transactions are not needed
        # Setting the isolation level on the engine, instead of the connection, avoids
        # a DBAPI statement to reset the transactional level back to the default before returning the
        # connection to the pool
        engine_args.setdefault("isolation_level", os.environ.get("CHALK_SQL_ISOLATION_LEVEL", "AUTOCOMMIT"))
        async_engine_args.setdefault("isolation_level", os.environ.get("CHALK_SQL_ISOLATION_LEVEL", "AUTOCOMMIT"))
        BaseSQLSource.__init__(self, name=name, engine_args=engine_args, async_engine_args=async_engine_args)
        self.connection = duckdb.connect(":memory:")
        for table in arrow_tables:
            # must be defined here to make the duckdb craziness work.
            t = arrow_tables[table]  # pyright: ignore [reportUnusedVariable]
            self.connection.execute(f"create table {table} as select * from t")

    def get_engine(self) -> Engine:
        raise NotImplementedError("DuckDB does not support async connections")

    def local_engine_url(self) -> URL:
        raise NotImplementedError("DuckDB does not support local_engine_url")
        # try:
        #     from sqlalchemy.engine.url import URL
        # except ImportError:
        #     raise missing_dependency_exception("chalkpy[duckdb]")
        # return URL.create(drivername="duckdb", database=self.file, query={"check_same_thread": "true"})

    def async_local_engine_url(self) -> URL:
        raise NotImplementedError("DuckDB does not support async connections")

    def get_sqlglot_dialect(self) -> Union[str, None]:
        return "duckdb"

    def _recreate_integration_variables(self) -> dict[str, str]:
        return {}

    def close(self):
        self.connection.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: chalk/kubernetes/v1/persistentvolume.proto
# Protobuf Python Version: 4.25.3
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n*chalk/kubernetes/v1/persistentvolume.proto\x12\x13\x63halk.kubernetes.v1"\xc6\x01\n\x1f\x43halkKubernetesPersistentVolume\x12L\n\x04spec\x18\x01 \x01(\x0b\x32\x38.chalk.kubernetes.v1.ChalkKubernetesPersistentVolumeSpecR\x04spec\x12U\n\x07metrics\x18\x02 \x01(\x0b\x32;.chalk.kubernetes.v1.ChalkKubernetesPersistentVolumeMetricsR\x07metrics"\xf2\x01\n#ChalkKubernetesPersistentVolumeSpec\x12#\n\rstorage_class\x18\x01 \x01(\tR\x0cstorageClass\x12\x12\n\x04name\x18\x02 \x01(\tR\x04name\x12!\n\x0c\x61\x63\x63\x65ss_modes\x18\x03 \x03(\tR\x0b\x61\x63\x63\x65ssModes\x12\x1a\n\x08\x63\x61pacity\x18\x04 \x01(\tR\x08\x63\x61pacity\x12\x16\n\x06status\x18\x05 \x01(\tR\x06status\x12%\n\x0ereclaim_policy\x18\x06 \x01(\tR\rreclaimPolicy\x12\x14\n\x05\x63laim\x18\x07 \x01(\tR\x05\x63laim"\x97\x01\n&ChalkKubernetesPersistentVolumeMetrics\x12%\n\x0e\x63\x61pacity_bytes\x18\x01 \x01(\x01R\rcapacityBytes\x12\x1d\n\nused_bytes\x18\x02 \x01(\x01R\tusedBytes\x12\'\n\x0f\x61vailable_bytes\x18\x03 \x01(\x01R\x0e\x61vailableBytesB\xba\x01\n\x17\x63om.chalk.kubernetes.v1B\x15PersistentvolumeProtoP\x01Z\x1akubernetes/v1;kubernetesv1\xa2\x02\x03\x43KX\xaa\x02\x13\x43halk.Kubernetes.V1\xca\x02\x13\x43halk\\Kubernetes\\V1\xe2\x02\x1f\x43halk\\Kubernetes\\V1\\GPBMetadata\xea\x02\x15\x43halk::Kubernetes::V1b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "chalk.kubernetes.v1.persistentvolume_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n\027com.chalk.kubernetes.v1B\025PersistentvolumeProtoP\001Z\032kubernetes/v1;kubernetesv1\242\002\003CKX\252\002\023Chalk.Kubernetes.V1\312\002\023Chalk\\Kubernetes\\V1\342\002\037Chalk\\Kubernetes\\V1\\GPBMetadata\352\002\025Chalk::Kubernetes::V1"
    _globals["_CHALKKUBERNETESPERSISTENTVOLUME"]._serialized_start = 68
    _globals["_CHALKKUBERNETESPERSISTENTVOLUME"]._serialized_end = 266
    _globals["_CHALKKUBERNETESPERSISTENTVOLUMESPEC"]._serialized_start = 269
    _globals["_CHALKKUBERNETESPERSISTENTVOLUMESPEC"]._serialized_end = 511
    _globals["_CHALKKUBERNETESPERSISTENTVOLUMEMETRICS"]._serialized_start = 514
    _globals["_CHALKKUBERNETESPERSISTENTVOLUMEMETRICS"]._serialized_end = 665
# @@protoc_insertion_point(module_scope)



from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import (
    ClassVar as _ClassVar,
    Iterable as _Iterable,
    Mapping as _Mapping,
    Optional as _Optional,
    Union as _Union,
)

DESCRIPTOR: _descriptor.FileDescriptor

class CDCTableReference(_message.Message):
    __slots__ = ("name", "schema")
    NAME_FIELD_NUMBER: _ClassVar[int]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    name: str
    schema: str
    def __init__(self, name: _Optional[str] = ..., schema: _Optional[str] = ...) -> None: ...

class CDCSource(_message.Message):
    __slots__ = ("integration_name", "tables")
    INTEGRATION_NAME_FIELD_NUMBER: _ClassVar[int]
    TABLES_FIELD_NUMBER: _ClassVar[int]
    integration_name: str
    tables: _containers.RepeatedCompositeFieldContainer[CDCTableReference]
    def __init__(
        self,
        integration_name: _Optional[str] = ...,
        tables: _Optional[_Iterable[_Union[CDCTableReference, _Mapping]]] = ...,
    ) -> None: ...



from google.protobuf import descriptor_pb2 as _descriptor_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Mapping as _Mapping, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor
ENCODING_FIELD_NUMBER: _ClassVar[int]
encoding: _descriptor.FieldDescriptor

class StringEncoding(_message.Message):
    __slots__ = ("mapping",)
    class MappingEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: int
        value: str
        def __init__(self, key: _Optional[int] = ..., value: _Optional[str] = ...) -> None: ...

    MAPPING_FIELD_NUMBER: _ClassVar[int]
    mapping: _containers.ScalarMap[int, str]
    def __init__(self, mapping: _Optional[_Mapping[int, str]] = ...) -> None: ...
